<?php

/**
 * BuddyServ IRC Services
 * 
 * Main configuration file
 * 
 * @name Config
 * @author Raymond Lynk <me@raymondj.ca>
 * @copyright (C) 2010, Raymond Lynk
 * @link http://buddyserv.raymondj.ca
 * @package Core
 * @version $Id: config.example.php 361 2009-12-18 06:57:40Z lynk $
 */

/*
 * 		 ______           __     __         _______                        
 * 		|   __ \.--.--.--|  |.--|  |.--.--.|     __|.-----.----.--.--.     
 * 		|   __ <|  |  |  _  ||  _  ||  |  ||__     ||  -__|   _|  |  |     
 * 		|______/|_____|_____||_____||___  ||_______||_____|__|  \___/      
 *  				    			   |_|                                 
 *                                                                         
 */

$config = array();

/* Specify your development folders (array)
 * This determines if we use the development database
 * or the real database */
$config['devfolders'] = array(
	'/home/pods/buddyserv1.4',
	'/home/pods/buddyserv1.5');

/* Specify production copies */
$config['prodfolders'] = array(
	'/home/pods/buddyserv1.4-release',
	'/home/pods/buddyserv1.5-release');

/* Specify our databases. We allow for 2 databases,
 * a production one which is your actual site database.
 * and a development one used for testing.
 */
/* Start with the production environment */
$config['database']['production'] = array(
	'NAME'		=> 'Live Database',
	'HOST'		=> '127.0.0.1',
	'USER'		=> 'mysqluser',
	'DB'		=> 'tbdev',
	'PASS'		=> 'letmein'
);

/* Now the development database */
$config['database']['development'] = array(
	'NAME'		=> 'Development Database',
	'HOST'		=> '127.0.0.1',
	'USER'		=> 'buddyserv',
	'DB'		=> 'tbdev2009',
	'PASS'		=> 'letmein'
);

/* We also allow for two IRC Servers. One for development,
 * and the other for production as well. Start with production.
 * 
 * You must modify your IRCD.conf file with plexus to allow
 * BuddyServ to connect to your network. Please ensure the information
 * matches what you have below.
 */
$config['irc']['production'] = array(
	'MYNAME'	=> 'buddyserv.yournetwork.ca',
	'HOSTNAME'	=> 'irc.yournetwork.ca',
	'PORT'		=> 6667,
	'PASSWORD'	=> 'password'
);

/* Development IRC Server */
$config['irc']['development'] = array(
	'MYNAME'	=> 'buddyserv.raymondj.ca',
	'HOSTNAME'	=> '192.168.56.102',
	'PORT'		=> 6665,
	'PASSWORD'	=> 'password'
);

/* In order for services to work properly, we need
 * to know your sites URL. We allow you to specify
 * a different url for both production and development.
 * 
 * NO TRAILING SLASH
 */
$config['site']['production']['URL'] = 'http://torrenturl.ca';
$config['site']['development']['URL'] = 'http://tbdev2009.pods';

/* Do you want to enable the RRD graphing? You MUST
 * have RRDTool installed, and you MUST create the database
 * before activating this feature. See the documentation
 * for help creating the database if you need it.
 * 
 * values are true or false
 */
$config['useRRD'] = true;

/* Where do you want the graphs to save? If you are
 * running on the same server and user account as your
 * website you can probably do something like 
 * '../public_html/yourtbdev/graphs/'
 */
$config['rrd']['production'] = '../tbdev2009/graphs/';
$config['rrd']['development'] = 'rrd/graphs/';


/* Begin network channel configuration.
 * The following special channels are available
 * for setting. Some commands and reporting functions
 * are restricted to listing in certain channels. You SHOULD
 * define all of these:
 * USER		= Main chat channel, lists new torrents
 * STAFF	= Staff channel
 * SYSOP	= System Operator channel
 * SUPPORT	= Support channel
 * CODER	= Coder debug channel and hangout
 */
$config['channels'] = array(
	'USER'		=> '#chat',
	'STAFF'		=> '#staff',
	'SYSOP'		=> '#sysop',
	'SUPPORT'	=> '#support',
	'CODER'		=> '#coders',
);

/* Start configuration of our service clients.
 * For the most part, the default values here should
 * suffice for most installations, however, changing
 * the hostname of the services is something you probably
 * WANT to do.
 */

/* Channel Management Service Client */
$config['clients']['chanserv'] = array(
			'nick'		=> 'ChanServ',
			'user'		=> 'buddyserv',
			'host'		=> 'services.irc',
			'name'		=> 'BuddyServ Channel Services',
			'modes'		=> '+pr');

/* Nickname Management Service Client */
$config['clients']['nickserv'] = array(
			'nick'		=> 'NickServ',
			'user'		=> 'buddyserv',
			'host'		=> 'services.irc',
			'name'		=> 'BuddyServ Nickname Services',
			'modes'		=> '+pr');

/* Operator Management Service Client */
$config['clients']['operserv'] = array(
			'nick'		=> 'OperServ',
			'user'		=> 'buddyserv',
			'host'		=> 'services.irc',
			'name'		=> 'BuddyServ Operator Services',
			'modes'		=> '+prio');

/* Memo/Mail Management Service Client */
$config['clients']['memoserv'] = array(
			'nick'		=> 'MemoServ',
			'user'		=> 'buddyserv',
			'host'		=> 'services.irc',
			'name'		=> 'BuddyServ Mail Services',
			'modes'		=> '+pr');

/* User statistics Service Client 
 * NOTE: The hostname of this client is also the base
 * hostname used for non-custom set vhost. You probably
 * WANT this to reflect your site domain. */
$config['clients']['userbot'] = array(
			'nick'		=> 'UserBot',
			'user'		=> 'buddyserv',
			'host'		=> 'services.irc',
			'name'		=> 'User Statistics Services',
			'modes'		=> '+prR');

/* Monitor client. Handles graphing and other server
 * updates. This is the OLD 'monitor' client.
 */
$config['clients']['buddyserv'] = array(
			'nick'		=> 'BuddyServ',
			'user'		=> 'buddyserv',
			'host'		=> 'services.irc',
			'name'		=> 'BuddyServ Core Client',
			'modes'		=> '+pro');

/* Flood protection service bot. Handles flood
 * issues.
 */
$config['clients']['floodserv'] = array(
			'nick'		=> 'FloodServ',
			'user'		=> 'buddyserv',
			'host'		=> 'services.irc',
			'name'		=> 'BuddyServ Flood Protection',
			'modes'		=> '+pir'
);

/* Flood protection basic limits, and activation flags 
 * Values are count:time, or 3:5 for 3 actions in 5 seconds 
 * ONLY messagePer IS CURRENTLY USED, BUT THE OTHERS MUST REMAIN
 * AS PARSING IS IN PLACE FOR FUTURE VERSIONS. */
$config['floodserv']['active'] = true; // use this service.

$config['floodserv']['joinsPer'] = '3:5';
$config['floodserv']['messagePer'] = '10:5';
$config['floodserv']['nicksPer'] = '3:10';

/* How long to GLine repeat offenders? In minutes */
$config['floodserv']['glineTime'] = 90;
$config['floodserv']['glineReason'] = 'Flood protection triggered.';

/* The following is the list of available
 * user classes on your site. Some of these
 * MUST exist to work properly. Those ones
 * have a comment prefixed, however, you
 * MUST change these values to match. If you
 * have more classes than is listed here, you
 * MAY add them as SITE_CLASS, however, services
 * will not do any special processing regarding them.
 */
define('SITE_USER',			0); // must exist
define('SITE_POWERUSER',	1); // must exist
define('SITE_VIP',			2); // must exist
define('SITE_UPLOADER',		3); // must exist
define('SITE_MODERATOR',	4); // must exist
define('SITE_ADMIN',		5); // must exist
define('SITE_SYSOP',		6); // must exist
define('SITE_CODER',		6); // must exist. Higher or equal to SYSOP


/* Language file settings. The following lists the available
 * languages, and also sets the default language for members.
 * We also have the option for a geo location lookup to automatically
 * decide which language we should present to a client when they connect.
 */

$config['languages']['available'] = array(
	'english'	=> true,
	'francais'	=> true,
	'espanol'	=> false);

/* Default language to use? */
$config['languages']['default'] = 'english';
/* Allow usage of geoip? */
$config['languages']['geoip'] = false;


?>