<?php

/**
 * BuddyServ IRC Services
 * 
 * Channel Service Event handler
 * 
 * @name ChanServ
 * @author Raymond Lynk <me@raymondj.ca>
 * @copyright (C) 2010, Raymond Lynk
 * @link http://buddyserv.raymondj.ca
 * @package Clients
 * @version $Id: Class.chanserv.php 361 2009-12-18 06:57:40Z lynk $
 */

/*
 * 		 ______           __     __         _______                        
 * 		|   __ \.--.--.--|  |.--|  |.--.--.|     __|.-----.----.--.--.     
 * 		|   __ <|  |  |  _  ||  _  ||  |  ||__     ||  -__|   _|  |  |     
 * 		|______/|_____|_____||_____||___  ||_______||_____|__|  \___/      
 *  				    			   |_|                                 
 *                                                                         
 */

class chanserv
{
	/* Access constants */
	const ACCESS_VOP	= 0x001;
	const ACCESS_HOP	= 0x002;
	const ACCESS_AOP	= 0x004;
	const ACCESS_SOP	= 0x008;
	const ACCESS_AKICK	= 0x010;
	
	/* ModeLock constants */
	const MLOCK_KEY				= 0x00000001;
	const MLOCK_NEGATE_KEY		= 0x00000002;
	const MLOCK_LIMIT			= 0x00000004;
	const MLOCK_NEGATE_LIMIT	= 0x00000008;
	const MLOCK_NOEXTERNAL		= 0x00000010;
	const MLOCK_NEGATE_NOEXTERNAL=0x00000020;
	const MLOCK_MODERATE		= 0x00000040;
	const MLOCK_NEGATE_MODERATE	= 0x00000080;
	const MLOCK_PRIVATE			= 0x00000100;
	const MLOCK_NEGATE_PRIVATE	= 0x00000200;
	const MLOCK_SECRET			= 0x00000400;
	const MLOCK_NEGATE_SECRET	= 0x00000800;
	const MLOCK_OPTOPIC			= 0x00001000;
	const MLOCK_NEGATE_OPTOPIC	= 0x00002000;
	const MLOCK_REGONLY			= 0x00004000;
	const MLOCK_NEGATE_REGONLY	= 0x00008000;
	const MLOCK_REGCHAT			= 0x00010000;
	const MLOCK_NEGATE_REGCHAT	= 0x00020000;
	const MLOCK_INVITEONLY		= 0x00040000;
	const MLOCK_NEGATE_INVITEONLY=0x00080000;
	const MLOCK_NOCOLORS		= 0x00100000;
	const MLOCK_NEGATE_NOCOLORS	= 0x00200000;
	const MLOCK_IRCOPONLY		= 0x00400000;
	const MLOCK_NEGATE_IRCOPONLY= 0x00800000;
	
	
	static $client;
	static $partchans = array();
	
	/**
	 * Attach the serviceclient object to $client
	 * and initialize the current service bot.
	 * 
	 * @param string $nick
	 * @param string $username
	 * @param string $host
	 * @param string $name
	 * @return void
	 */
	static function initialize($nick, $username, $host, $name, $modes)
	{
		self::$client = new serviceclient();
		self::$client->initialize($nick, $username, $host, $name, $modes);
	}
	
	/**
	 * Magic getter
	 *
	 * @param string $what
	 * @return mixed
	 */
	function __get($what)
	{
		if (self::$client->{$what})
			return self::$client->{$what};
			
	}
	
	/**
	 * Introduce the client to the uplink server.
	 * 
	 * @return void
	 */
	static function introduce()
	{
		self::$client->introduce();
	}
	
	/**
	 * Parse channel commands for this service client
	 * 
	 * @param string $nick
	 * @param string $channel
	 * @param string $arguments
	 * @return void
	 */
	static function parser_channel($nick, $channel, $arguments)
	{	
		$checkuser = user::getuser($nick);
		
		if ($checkuser->forbid)
			return;
			
		if (($checkuser->modes & UMODE_REGISTERED) == 0)
			return;
			
		if (strcasecmp($arguments[0], '.BINARY') == 0)
			return chanserv::binarymode($nick, $arguments[1]);
		
		if (strcasecmp($arguments[0], '.time') == 0)
			return chanserv::time($nick, $channel);
			
		if (strcasecmp($arguments[0], '.KB') == 0)
			return chanserv::kickban($nick, $channel, $arguments);
				
		if (strcasecmp($arguments[0], '.OP') == 0)
			return chanserv::opDeop($nick, $channel, $arguments[0], 'add', CMODE_OP, $arguments[1]);
			
		if (strcasecmp($arguments[0], '.DEOP') == 0)
			return chanserv::opDeop($nick, $channel, $arguments[0], 'remove', CMODE_OP, $arguments[1]);
			
		if (strcasecmp($arguments[0], '.HALFOP') == 0)
			return chanserv::opDeop($nick, $channel, $arguments[0], 'add', CMODE_HALFOP, $arguments[1]);
			
		if (strcasecmp($arguments[0], '.DEHALFOP') == 0)
			return chanserv::opDeop($nick, $channel, $arguments[0], 'remove', CMODE_HALFOP, $arguments[1]);
			
		if (strcasecmp($arguments[0], '.VOICE') == 0)
			return chanserv::opDeop($nick, $channel, $arguments[0], 'add', CMODE_VOICE, $arguments[1]);
			
		if (strcasecmp($arguments[0], '.DEVOICE') == 0)
			return chanserv::opDeop($nick, $channel, $arguments[0], 'remove', CMODE_VOICE, $arguments[1]);

		if (strcasecmp($arguments[0], '.PROTECT') == 0)
			return chanserv::opDeop($nick, $channel, $arguments[0], 'add', CMODE_PROTECT, $arguments[1]);
			
		if (strcasecmp($arguments[0], '.DEPROTECT') == 0)
			return chanserv::opDeop($nick, $channel, $arguments[0], 'remove', CMODE_PROTECT, $arguments[1]);
			
		if (strcasecmp($arguments[0], '.FOUNDER') == 0)
			return chanserv::opDeop($nick, $channel, $arguments[0], 'add', CMODE_FOUNDER, $arguments[1]);
			
		if (strcasecmp($arguments[0], '.DEFOUNDER') == 0)
			return chanserv::opDeop($nick, $channel, $arguments[0], 'remove', CMODE_FOUNDER, $arguments[1]);
			
			
		if ((!strcasecmp($arguments[0], '.SOP')) or (!strcasecmp($arguments[0], '.AOP')) 
			or (!strcasecmp($arguments[0], '.HOP')) or (!strcasecmp($arguments[0], '.VOP')))
		{
			return chanserv::modaccess($nick, $arguments, $channel);
		}
		
		if (strcasecmp($arguments[0], '.HELP') == 0)
		{
			array_shift($arguments);
			
			if (count($arguments) == 0)
				return chanserv::$client->help($nick, 'chanserv');
			
			return chanserv::$client->help($nick, 'chanserv', $arguments);
		}
	}

	/**
	 * Returns current server time in two formats.
	 * 
	 * @param string $nick
	 * @param string $channel
	 * @return void
	 */
	static function time($nick, $channel)
	{
		$time = current_time();
		$time2 = strftime("%d/%b/%Y %H:%M:%S",$time);
		
		$time3 = time();
		$time4 = strftime("%d/%b/%Y %H:%M:%S",$time3);
		
		self::$client->msg($channel, "Local: {$time3} = {$time4} / Server: {$time} = {$time2}");
	}
	
	/**
	 * Kickban command to remove a user from a channel
	 * 
	 * @param string $nick
	 * @param string $channel
	 * @param string $arguments
	 * @return void
	 */
	static function kickban($nick, $channel, $arguments)
	{
		global $irc, $me;
		
		array_shift($arguments);
		$who = array_shift($arguments);
		$reason = implode(' ', $arguments);
		
		if (!$who)
			$who = $nick;
			
		$user = user::getuser($nick);
		$check_user = user::getuser($who);
		
		if (!$reason)
			$reason = $check_user->language->CS_USERNOWBANNED;
		
		if (!channel::exists($channel))
			return self::$client->notice($nick, sprintf($user->language->CS_NOSUCHCHANNEL, $channel));
			
		$chan = channel::get($channel);
		
		if (!($chan->has_access($nick, chanserv::ACCESS_HOP) or $chan->has_access($nick, chanServ::ACCESS_AOP) or $chan->has_access($nick, chanServ::ACCESS_SOP) or $chan->founder == $nick))
		{
			return;
		}
		
		if (!$check_user)
			return self::$client->notice($nick, sprintf($user->language->CS_NOUSERONIRC, $who));
			
		self::$client->chanMode($channel, '+b *!*@'.(($check_user->vhost) ? $check_user->vhost : $check_user->host));
		self::$client->kick($channel, $who, $reason);
	
	}
	
	/**
	 * Parse commands sent directly to this service.
	 * 
	 * @param string $nick
	 * @param string $arguments
	 * @return void
	 */
	static function parser($nick, $arguments)
	{
		$checkuser = user::getuser($nick);
		
		if ($checkuser->forbid)
			return self::$client->notice($nick, $checkuser->language->CS_DISABLEDACCOUNT);
			
		if (($checkuser->modes & UMODE_REGISTERED) == 0)
			return self::$client->notice($nick, sprintf($checkuser->language->CS_NOTIDENTIFIED, nickserv::$client->nick));

		/* A SET command */
		if (strcasecmp($arguments[0], 'SET') == 0)
		{
			if (strcasecmp($arguments[1], 'RESTRICT') == 0)
				return chanserv::setRestrict($nick, $arguments[2], $arguments[3]);
				
			if (strcasecmp($arguments[1], 'STRICT') == 0)
				return chanserv::setSecure($nick, $arguments[2], $arguments[3]);
				
	//		if (strcasecmp($arguments[0], 'MLOCK') == 0)
	//			return chanserv::setMlock($nick, $arguments[2], $arguments[3]);
			
				
			else
				return chanserv::unknowncmd($nick, 'SET '.$arguments[1]);
				
		}
			
		if (strcasecmp($arguments[0], 'CLEAR') == 0)
			return chanserv::clear($nick, $arguments[1], $arguments[2]);
			
		if (strcasecmp($arguments[0], 'REGISTER') == 0)
			return chanserv::register($nick, $arguments);
			
		if (strcasecmp($arguments[0], 'FORBID') == 0)
			return chanserv::forbid($nick, $arguments);
		
		if (strcasecmp($arguments[0], 'GETKEY') == 0)
			return chanserv::getKey($nick, $arguments[1]);
		
		if (strcasecmp($arguments[0], 'INFO') == 0)
			return chanserv::info($nick, $arguments[1]);
			
		if (strcasecmp($arguments[0], 'INVITE') == 0)
			return chanserv::invite($nick, $arguments[1]);
			
		if (strcasecmp($arguments[0], 'BINARY') == 0)
			return chanserv::binarymode($nick, $arguments[1]);
			
		if (strcasecmp($arguments[0], 'ALLOW') == 0)
			return chanserv::allow($nick, $arguments[1], $arguments[2]);
			
		if (strcasecmp($arguments[0], 'OP') == 0)
			return chanserv::opDeop($nick, $arguments[1], $arguments[0], 'add', CMODE_OP, $arguments[2]);
			
		if (strcasecmp($arguments[0], 'DEOP') == 0)
			return chanserv::opDeop($nick, $arguments[1], $arguments[0], 'remove', CMODE_OP, $arguments[2]);

		if (strcasecmp($arguments[0], 'UNBAN') == 0)
			return chanserv::unban($nick, $arguments[1]);
		
		if (strcasecmp($arguments[0], 'DROP') == 0)
			return chanserv::drop($nick, $arguments[1]);
			
		if (strcasecmp($arguments[0], 'VOICE') == 0)
			return chanserv::opDeop($nick, $arguments[1], $arguments[0], 'add', CMODE_VOICE, $arguments[2]);
			
		if (strcasecmp($arguments[0], 'DEVOICE') == 0)
			return chanserv::opDeop($nick, $arguments[1], $arguments[0], 'remove', CMODE_VOICE, $arguments[2]);
	
		if (strcasecmp($arguments[0], 'HALFOP') == 0)
			return chanserv::opDeop($nick, $arguments[1], $arguments[0], 'add', CMODE_HALFOP, $arguments[2]);
			
		if (strcasecmp($arguments[0], 'DEHALFOP') == 0)
			return chanserv::opDeop($nick, $arguments[1], $arguments[0], 'remove', CMODE_HALFOP, $arguments[2]);
		
		if (strcasecmp($arguments[0], 'PROTECT') == 0)
			return chanserv::opDeop($nick, $arguments[1], $arguments[0], 'add', CMODE_PROTECT, $arguments[2]);
			
		if (strcasecmp($arguments[0], 'DEPROTECT') == 0)
			return chanserv::opDeop($nick, $arguments[1], $arguments[0], 'remove', CMODE_PROTECT, $arguments[2]);
			
		if (strcasecmp($arguments[0], 'FOUNDER') == 0)
			return chanserv::opDeop($nick, $arguments[1], $arguments[0], 'add', CMODE_FOUNDER, $arguments[2]);
			
		if (strcasecmp($arguments[0], 'DEFOUNDER') == 0)
			return chanserv::opDeop($nick, $arguments[1], $arguments[0], 'remove', CMODE_FOUNDER, $arguments[2]);
			
		/* the following is a doozy. If the command is an ACCESS modification or listing
		 * we call chanserv_modaccess */
		if ((!strcasecmp($arguments[0], 'SOP')) or (!strcasecmp($arguments[0], 'AOP')) 
			or (!strcasecmp($arguments[0], 'HOP')) or (!strcasecmp($arguments[0], 'VOP')))
		{
			return chanserv::modaccess($nick, $arguments);
		}
		
		if (!strcasecmp($arguments[0], 'AKICK'))
		{
			return chanserv::akick($nick, $arguments);
		}
		
		if (strcasecmp($arguments[0], 'HELP') == 0)
		{
			array_shift($arguments);
			
			if (count($arguments) == 0)
				return chanserv::$client->help($nick, 'chanserv');
			
			return chanserv::$client->help($nick, 'chanserv', $arguments);
		}
			
		return chanserv::unknowncmd($nick, $arguments[0]);
	}
	
	/**
	 * Process requests for the AKICK list.
	 * 
	 * @param string $nick
	 * @param array $arguments
	 * @return void
	 */
	public static function akick($nick, $arguments)
	{
		global $language;
		
		array_shift($arguments);
		$channel = array_shift($arguments);
		$function = array_shift($arguments);
		
		$userObj = user::getuser($nick);
		$chanObj = channel::get($channel);
		
		/* channel doesn't exist? */
		if (!($chanObj instanceof channel))
		{
			return self::$client->notice($nick, sprintf($userObj->language->CS_NOSUCHCHANNEL, $channel));
		}
		
		/* channel isn't registered? */
		if (!$chanObj->active)
			return self::$client->notice($nick, sprintf($userObj->language->CS_NOTREGISTERED, $channel));
			
		if (!strcasecmp($function, 'DEL'))
		{
			if ((!$chanObj->has_access($nick, chanserv::ACCESS_SOP)) and (($chanObj->founder != $nick)))
				return self::denied($nick);
				
			if (count($arguments) == 0)
				return self::$client->notice($nick, 'Syntax: AKICK <#channel> DEL <user|mask>');
				
			$mask = array_shift($arguments);
			
			if (!$chanObj->akickExists($mask))
				return self::$client->notice($nick, sprintf($userObj->language->CS_ACCESSDELNOFOUND, $mask, $channel, 'AKICK'));
				
			if ($chanObj->akickDelete($mask))
				return self::$client->notice($nick, sprintf($userObj->language->CS_ACCESSDEL, $mask, $channel, 'AKICK'));
				
			/* not found in akick list */
			return self::$client->notice($nick, sprintf($userObj->language->CS_ACCESSDELNOFOUND, $mask, $channel, 'AKICK'));
			
		}
			
		elseif (!strcasecmp($function, 'ADD'))
		{
			if (count($arguments) == 0)
				return self::$client->notice($nick, 'Syntax: AKICK <#channel> ADD <user|mask> [reason]');
				
			if ((!$chanObj->has_access($nick, chanserv::ACCESS_SOP)) and (($chanObj->founder != $nick)))
				return self::denied($nick);
				
			$mask = array_shift($arguments);
			
			/* Check for a valid reason, or set a default. */
			if (count($arguments) == 0)
				$reason = $language->CS_USERNOWBANNED;
				
			else
				$reason = implode(' ', $arguments);
				
			/* Check if the AKICK already exists. */
			if ($chanObj->akickExists($mask))
				return self::$client->notice($nick, sprintf($userObj->language->CS_ACCESSEXISTS, $mask, $channel));
				
			/* if its a user, ensure its a registered one. */
			if (strpos($mask, '@') === false)
			{
				if (!user::ismember($mask))
					return self::$client->notice($nick, sprintf($userObj->language->NS_USERNOTREGISTERED, $mask));
					
			}
			
			if ($chanObj->akickAdd($mask, $nick, $reason))
				return self::$client->notice($nick, sprintf($userObj->language->CS_ACCESSADDED, $mask, $channel, 'AKICK'));
				
			else
				return self::$client->notice($nick, $userObj->language->MS_INTERNALERROR);
		}
			
		elseif (!strcasecmp($function, 'LIST'))
		{
			/* If not SOP, deny the request. */
			if ((!$chanObj->has_access($nick, chanserv::ACCESS_SOP)) and (($chanObj->founder != $nick)))
				return self::denied($nick);
				
			self::$client->notice($nick, sprintf($userObj->language->CS_ACCESSLISTSTART, 'AKICK', $channel));
			
			if (!is_array($chanObj->akick))
				return self::$client->notice($nick, $userObj->language->CS_ACCESSLISTEND);
				
			$i = 1;
				
			foreach ($chanObj->akick as $mask => $array)
			{
				self::$client->notice($nick, $i.') '.$mask.' ['.$array['reason'].'] by '.$array['user']);
				$i++;
			}
			
			return self::$client->notice($nick, $userObj->language->CS_ACCESSLISTEND);
		}
		
		else
			return self::unknowncmd($nick, 'AKICK '.$function);
	}
	
	/**
	 * Drops a registered channel from management.
	 * 
	 * @param string $nick
	 * @param string $channel
	 * @return void
	 */
	public static function drop($nick, $channel)
	{
		global $db;
		
		$userObj = user::getuser($nick);
		
		if (!channel::exists($channel))
			$chanObj = new channel($channel);
			
		else
			$chanObj = channel::get($channel);
			
		if (!$chanObj->active)
			return self::$client->notice($nick, sprintf($userObj->language->CS_NOTREGISTERED, $channel));
			
		if (($chanObj->founder != $nick) and ($userObj->level < SITE_SYSOP))
			return self::denied($nick);
			
		$query = $db->query(sprintf("DELETE FROM `buddyserv_channels` WHERE `name` = '%s' LIMIT 1", mysql_real_escape_string($channel)));
		
		if ($db->num_affected($query) == 0)
			return self::$client->notice($nick, sprintf($userObj->language->CS_FAILEDDROP, $channel));
			
		$chanObj->active = false;
		$chanObj->secureops = false;
		$chanObj->access = array();
		$chanObj->akick = array();
		$chanObj->founder = null;
		$chanObj->restricted = false;
		$chanObj->mlock = 0x000;
		self::$client->notice($nick, sprintf($userObj->language->CS_SUCCESSDROP, $channel));
		
		if (count($chanObj->users) > 0)
		{
			self::$client->part($channel, 'DROP by '.$nick);
		}
		
	}
	
	/**
	 * Sends information to a user about a channel
	 * 
	 * @param string $nick
	 * @param string $channel
	 * @return void
	 */
	public static function info($nick, $channel)
	{
		$userObj = user::getuser($nick);
		
		if (!channel::exists($channel))
			$chanObj = new channel($channel);
		
		else
			$chanObj = channel::get($channel);
			
		if (!$chanObj->active)
			return self::$client->notice($nick, sprintf($userObj->language->CS_NOTREGISTERED, $channel));
			
		if ($chanObj->restricted)
			$options = 'Restricted, ';
			
		if ($chanObj->secureops)
			$options .= 'SecureOPs, ';
			
		self::$client->notice($nick, sprintf($userObj->language->CS_INFO_START, $channel));
		self::$client->notice($nick, sprintf($userObj->language->CS_INFO_FOUNDER, $chanObj->founder));
		self::$client->notice($nick, sprintf($userObj->language->CS_INFO_DESCRIPTION, $chanObj->description));
		self::$client->notice($nick, sprintf($userObj->language->CS_INFO_REGISTERED, strftime("%c", $chanObj->regdate)));
		self::$client->notice($nick, sprintf($userObj->language->CS_INFO_LASTUSED, strftime("%c", $chanObj->lastused)));
		self::$client->notice($nick, sprintf($userObj->language->CS_INFO_SETTINGS, $options));
		
		if (($userObj->level > SITE_MODERATOR) or ($chanObj->founder == $nick) or ($chanObj->has_access($nick, chanserv::ACCESS_VOP)))
		{
			self::$client->notice($nick, "-");
			self::$client->notice($nick, sprintf($userObj->language->CS_INFO_TOPIC, $chanObj->topic));
		//	self::$client->notice($nick, "      Modes: <unknown>");
			self::$client->notice($nick, sprintf($userObj->language->CS_INFO_USERS, count($chanObj->users)));
		}
		
		return self::$client->notice($nick, $userObj->language->CS_INFO_END);
		
	}
	
	/**
	 * Invite a user to the channel.
	 * 
	 * @param string $nick
	 * @param string $channel
	 * @return unknown_type
	 */
	public static function invite($nick, $channel)
	{
		global $irc, $me;
		
		$userObj = user::getuser($nick);
		
		if (!channel::exists($channel))
			return self::$client->notice($nick, sprintf($userObj->language->CS_NOSUCHCHANNEL, $channel));
			
		$chanObj = channel::get($channel);
		
		if (!$chanObj->active)
			return self::$client->notice($nick, sprintf($userObj->language->CS_NOTREGISTERED, $channel));
			
		if ((!$chanObj->has_access($nick, chanserv::ACCESS_VOP) and (!$chanObj->founder != $nick) and ($userObj->level < SITE_ADMIN)))
			return self::denied($nick);
			
		if ($chanObj->isMember($nick))
			return self::$client->notice($nick, sprintf($userObj->language->CS_ALREADYMEMBER, $channel));
			
		$irc->send(sprintf(INVITE, self::$client->nick, $nick, $channel, $chanObj->ctime));
		self::$client->notice($nick, sprintf($userObj->language->CS_INVITED, $channel));
		
		
	}
	
	/**
	 * Clear channel type command to remove users,
	 * modes and operator status's.
	 * 
	 * @param string $nick
	 * @param string $channel
	 * @param string $what
	 * @return void
	 */
	public static function clear($nick, $channel, $what)
	{
		global $chanusers, $language;
		
		$user = user::getuser($nick);
		
		if (!channel::exists($channel))
			return self::$client->notice($nick, sprintf($user->language->CS_NOSUCHCHANNEL, $channel));
			
		$chanObj = channel::get($channel);
		
		if (!$chanObj->active)
			return self::$client->notice($nick, sprintf($user->language->CS_NOTREGISTERED, $channel));
			
		if (($chanObj->founder == $nick) or ($chanObj->has_access($nick, chanserv::ACCESS_SOP)))
		{
			/* has access */
			if (strcasecmp($what, 'users') == 0)
			{
				/* Kick all users out of the channel */
				$allmembers = $chanObj->users;
				
				self::$client->notice($channel, sprintf($language->CS_CLEAR_USERS, $nick));
				self::$client->chanMode($channel, '+b *!*@*');
				
				foreach ($allmembers as $each => $array)
				{
					$user = user::getuser($each);
					
					if ($user->level > SITE_ADMIN)
					{
						continue;
					}
					
					self::$client->kick($channel, $each, sprintf($language->CS_CLEAR_USERS, $nick));
				}
				
				self::$client->chanMode($channel, '-b *!*@*');
				return self::$client->notice($nick, sprintf($user->language->CS_ALL_USERS_CLEARED, $channel));
			}
			
			elseif (strcasecmp($what, 'ops') == 0)
			{
				self::$client->notice($channel, sprintf($language->CS_CLEAR_OPS, $nick));
				
				$chanusers = $chanObj->users;
				
				foreach ($chanusers as $eachuser => $array)
				{
					$modes = array();
					$string = null;

					if ($chanObj->hasUserStatus($eachuser, CMODE_FOUNDER))
					{
						$modes[] = 'q';
						$params[] = $eachuser;
					}
						
					if ($chanObj->hasUserStatus($eachuser, CMODE_PROTECT))
					{
						$modes[] = 'a';
						$params[] = $eachuser;
					}
						
					if ($chanObj->hasUserStatus($eachuser, CMODE_OP))
					{
						$modes[] = 'o';
						$params[] = $eachuser;
					}
						
					if ($chanObj->hasUserStatus($eachuser, CMODE_HALFOP))
					{
						$modes[] = 'h';
						$params[] = $eachuser;
					}
					
					if ($chanObj->hasUserStatus($eachuser, CMODE_VOICE))
					{
						$modes[] = 'v';
						$params[] = $eachuser;
					}
						
					/* end mode checks */
					if ((count($modes) == 0) or (count($params)) == 0)
						continue;
						
					$string = implode(" ", $params);
					$modes = '-'.implode('-', $modes);
				
					self::$client->chanMode($channel, $modes.' '.$string);
					
					$params = array();
					$modes = array();
				}
				
				return self::$client->notice($nick, sprintf($user->language->CS_ALL_OPS_CLEARED, $channel));
				
			}
			
			elseif (strcasecmp($what, 'modes') == 0)
			{
				self::$client->chanMode($channel, '+nt-imspklR *');
				
				$bans = $chanObj->bans;
				
				foreach ($bans as $each => $added)
					self::$client->chanMode($channel, '-b '.$each);
					
				return self::$client->notice($nick, sprintf($user->language->CS_ALL_MODES_CLEARED, $channel));
			}
			
		}
		
		return self::denied($nick);
	}
	
	/**
	 * Command to set the secureops option
	 * of this channel.
	 * 
	 * @param string $nick
	 * @param string $channel
	 * @param [on|off] $onoff
	 * @return void
	 */
	public static function setSecure($nick, $channel, $onoff)
	{
		global $db;
		
		$userObj = user::getuser($nick);
		
		if (!channel::exists($channel))
			return self::$client->notice($nick, sprintf($userObj->language->CS_NOSUCHCHANNEL, $channel));
			
		$chanObj = channel::get($channel);
		
		if (!$chanObj->active)
			return self::$client->notice($nick, sprintf($userObj->language->CS_NOTREGISTERED, $channel));
			
		/* Only founders and Admin+ can set this value */
		if (($chanObj->founder != $nick) and ($userObj->level < SITE_ADMIN))
			return self::denied($nick);
			
		switch ($onoff)
		{
			case 'on':
				$onoff = 'on';
				$flag = true;
				break;
			
			default:
				$onoff = 'off';
				$flag = false;
				break;
		}
		
		$chanObj->secureops = $flag;
		
		$query = $db->query(sprintf("UPDATE `buddyserv_channels` SET `secureops` = '%s' WHERE `id` = %d LIMIT 1", $onoff, $chanObj->id));
		
		if ($db->num_affected($query) == 0)
			return self::$client->notice($nick, sprintf($userObj->language->CS_NOCHANGE, 'STRICT Ops', $channel));

		return self::$client->notice($nick, sprintf($userObj->language->CS_STRICT_SET, $channel, strtoupper($onoff)));

	}
	
	/**
	 * Command to set the restrict status on a channel
	 * 
	 * @param string $nick
	 * @param string $channel
	 * @param [on|off] $onoff
	 * @return void
	 */
	public static function setRestrict($nick, $channel, $onoff)
	{
		global $db;
					
		$userObj = user::getuser($nick);
		
		if (!channel::exists($channel))
			return self::$client->notice($nick, sprintf($userObj->language->CS_NOSUCHCHANNEL, $channel));

		$chanObj = channel::get($channel);
		
		if (!$chanObj->active)
			return self::$client->notice($nick, sprintf($userObj->language->CS_NOTREGISTERED, $channel));
			
		/* Only founders and Admin+ can set this value */
		if (($chanObj->founder != $nick) and ($userObj->level < SITE_ADMIN))
			return self::denied($nick);
			
		switch ($onoff)
		{
			case 'on':
				$onoff = 'on';
				$flag = true;
				break;
			
			default:
				$onoff = 'off';
				$flag = false;
				break;
		}
		
		$chanObj->restricted = $flag;
		
		$query = $db->query(sprintf("UPDATE `buddyserv_channels` SET `restricted` = '%s' WHERE `id` = %d LIMIT 1", $onoff, $chanObj->id));
		
		if ($db->num_affected($query) == 0)
			return self::$client->notice($nick, sprintf($userObj->language->CS_NOCHANGE, 'RESTRICT', $channel));

		return self::$client->notice($nick, sprintf($userObj->language->CS_RESTRICT_SET, $channel, strtoupper($onoff)));

	}
	
	/**
	 * Unban command to remove all matching
	 * bans on a the issuer.
	 * 
	 * @param string $nick
	 * @param string $channel
	 * @return void
	 */
	public static function unban($nick, $channel)
	{
		$userObj = user::getuser($nick);
		
		if (!channel::exists($channel))
			return self::$client->notice($nick, sprintf($userObj->language->CS_NOSUCHCHANNEL, $channel));
			
		$chanObj = channel::get($channel);
		
		if (!$chanObj->active)
			return self::$client->notice($nick, sprintf($userObj->language->CS_NOTREGISTERED, $channel));
		
		if (($chanObj->has_access($nick, chanserv::ACCESS_HOP)) or ($userObj->level > SITE_MODERATOR))
		{
			if (!($chanObj->isBanned($userObj->fullMask()) or ($chanObj->isBanned($userObj->fullMask(true)))))
			{
				/* Channel says not banned. */
				return self::$client->notice($nick, sprintf($userObj->language->CS_NOBANSMATCHING, $channel));
			}
			
			$total=0;
			
			/* check virtual host mask first */
			while ($each = $chanObj->findMatchingBan($userObj->fullMask(true)))
			{
				self::$client->chanMode($channel, '-b '. $each);
				$total++;
			}
			
			while ($each = $chanObj->findMatchingBan($userObj->fullMask()))
			{
				self::$client->chanMode($channel, '-b '. $each);
				$total++;
			}
			
			return self::$client->notice($nick, sprintf($userObj->language->CS_BANSMATCHINGREMOVED, $total, $channel));
			
		}
		
		return self::denied($nick);
		
	} 
	
	/**
	 * Processing OP/DEOP/HALFOP ect access
	 * requests.
	 * 
	 * @param string $nick
	 * @param string $channel
	 * @param string $command
	 * @param add|remove $type
	 * @param bit $mode
	 * @param string $target
	 * @return void
	 */
	public static function opDeop($nick, $channel, $command, $type, $mode, $target=false)
	{
		$user = user::getuser($nick);
		
		if (!$channel)
			return self::$client->notice($nick, 'SYNTAX: '.$command.' <#channel> <who>');
			
		if (!channel::exists($channel))
			return self::$client->notice($nick, sprintf($user->language->CS_NOSUCHCHANNEL, $channel));
			
		$chanObject = channel::get($channel);
		
		if (!$chanObject->active)
			return self::$client->notice($nick, sprintf($user->language->CS_NOTREGISTERED, $channel));
			
		if ($target)
			$targetObject = user::getuser($target);
		
		else
		{
			$target = $nick;
			$targetObject = &$user;
		}
			
		if (!($targetObject instanceof user))
			return self::$client->notice($nick, sprintf($user->language->CS_NOUSERONIRC, $target));

		if (!$chanObject->isMember($target))
			return self::$client->notice($nick, sprintf($user->language->CS_NOUSERONCHAN, $target, $channel));
			
		/* Permission checking/ */
		if (($chanObject->founder == $nick) or ($user->level > SITE_ADMIN))
		{
			// no processing is needed, we are allowed permission.
		}
		
		elseif ($chanObject->has_access($nick, chanserv::ACCESS_SOP))
		{
			// no processing is needed, we are allowed permission.
		}
		
		elseif (($mode == CMODE_FOUNDER) and ($chanObject->founder != $nick))
			return self::denied($nick);
		
		elseif (($mode == CMODE_PROTECT) and (!$chanObject->has_access($nick, chanserv::ACCESS_SOP)))
			return self::denied($nick);
			
		elseif (($mode == CMODE_OP) and (!$chanObject->has_access($nick, chanserv::ACCESS_AOP)))
			return self::denied($nick);
		
		elseif ((($mode == CMODE_HALFOP) and (!$chanObject->has_access($nick, chanserv::ACCESS_HOP)) or ($nick != $target)))
			return self::denied($nick);
			
		elseif ((($mode == CMODE_VOICE) and (!$chanObject->has_access($nick, chanserv::ACCESS_VOP)) or ($nick != $target)))
			return self::denied($nick);
			
		switch ($mode)
		{
			case CMODE_OP:
				$letter = 'o';
				break;
			
			case CMODE_HALFOP:
				$letter = 'h';
				break;
				
			case CMODE_VOICE:
				$letter = 'v';
				break;
				
			case CMODE_PROTECT:
				$letter = 'a';
				break;
				
			case CMODE_FOUNDER:
				$letter = 'q';
				break;
		}
		
		if ($type == 'remove')
		{
			if ($chanObject->hasUserStatus($target, $mode))
			{
				self::$client->chanMode($channel, sprintf("-%s %s",
					$letter,
					$target));

				if (substr($command,0,1) == '.')
					return;
					
				return self::$client->notice('@'.$channel, $nick." used ".strtoupper($command)." ".$target);
			}
			
			else
				return self::$client->notice($nick, sprintf($user->language->CS_USERHASNOTMODE, $target, $channel));
				
		}
		
		elseif ($type == 'add')
		{
			if (!$chanObject->hasUserStatus($target, $mode))
			{
				self::$client->chanMode($channel, sprintf("+%s %s",
					$letter,
					$target));

				if (substr($command,0,1) == '.')
					return;
					
				return self::$client->notice('@'.$channel, $nick." used ".strtoupper($command)." ".$target);
			}
			
			else
				return self::$client->notice($nick, sprintf($user->language->CS_USERHASMODE, $target, $channel));
			
		}
		
	}
	
	/**
	 * Sends the user the channel key
	 * set on a channel.
	 * 
	 * @param string $nick
	 * @param string $channel
	 * @return void
	 */
	public static function getKey($nick, $channel)
	{
		$userObject = user::getuser($nick);

		if (!channel::exists($channel))
			return self::$client->notice($nick, sprintf($userObject->language->CS_NOSUCHCHANNEL, $channel));

		$chanObject = channel::get($channel);
		
		if (($userObject->level < SITE_ADMIN) and (!$chanObject->has_access($nick, chanserv::ACCESS_VOP)))
			return self::denied($nick);
			
		/* is there a key set? */
		if ($chanObject->key)
			return self::$client->notice($nick, sprintf($userObject->language->CS_KEYIS, $channel, $chanObject->key));
			
		else
			return self::$client->notice($nick, sprintf($userObject->language->CS_NOKEYSET, $channel));
	}
	
	/**
	 * Leave channels that services should no longer
	 * be occupying for one reason or another.
	 * 
	 * @return void
	 */
	static function leave_expired_channels()
	{
		if (count(self::$partchans) > 0)
		{
			foreach (self::$partchans as $each => $when)
			{
				/* check if the timestamp is old enough to part */
				if ($when < time())
				{
					self::$client->part($each);
					unset(self::$partchans[$each]);
				}
			}
		}
	}
	
	/**
	 * Handle topic changes, save to database.
	 * 
	 * @param string $nick
	 * @param string $channel
	 * @param string $topic
	 * @return void
	 */
	static function handletopic($nick, $channel, $topic)
	{
		global $db;
		
		$chan = channel::get($channel);
		
		if ($chan->active)
		{
			$chan->topic = $topic;
			$topic = mysql_escape_string($topic);
			
			$db->query("UPDATE `buddyserv_channels` SET `topic` = '{$topic}' WHERE `name` = '".mysql_real_escape_string($chan->name)."' LIMIT 1");
		}
	}
	
	/**
	 * Forbid a channel from being used by non-staff.
	 * 
	 * @param string $nick
	 * @param string $arguments
	 * @return void
	 */
	static function forbid($nick, $arguments)
	{
		global $irc, $db, $me, $language;
		
		$user = user::getuser($nick);
		
		array_shift($arguments);
		$dowhat = array_shift($arguments);
		$channel = mysql_real_escape_string(array_shift($arguments));
		$reason = mysql_real_escape_string(implode(' ',$arguments));
			
			
		if (($user->level < SITE_ADMIN) or (!$user->identified()))
		{
			self::denied($nick);
			return $irc->send(sprintf(OPERWALL, self::$client->nick, sprintf($language->CS_DENIEDACCESSTELL, 'FORBID '.$channel, $nick)));
		}

			if (stristr($dowhat, 'ADD'))
			{
				$query = $db->query("SELECT `reason` FROM `buddyserv_jupedchans` WHERE `channel` = '".$channel."' LIMIT 1");
				
				if ($db->num_rows($query) > 0)
					return self::$client->notice($nick, sprintf($user->language->CS_ALREADY_FORBID, $channel));
			
				else 
				{
					if (!$reason)
						$reason = 'No reason specified.';
						
					$db->query("INSERT INTO `buddyserv_jupedchans` (`channel`, `reason`) VALUES ('".$channel."', '".$reason."')");
					self::$client->notice($nick, sprintf($user->language->CS_NOW_FORBIDEN, $channel));
					
					return $irc->send(sprintf(OPERWALL, self::$client->nick, sprintf($language->CS_FORBIDWALL, $channel, $nick, $reason)));
				}
			}
			
			elseif (stristr($dowhat, 'DEL'))
			{
				$query = $db->query("SELECT `reason` FROM `buddyserv_jupedchans` WHERE `channel` = '".$channel."' LIMIT 1");
				
				if ($db->num_rows($query) == 0)
					return self::$client->notice($nick, sprintf($user->language->CS_NOT_FORBIDEN, $channel));
					
				else 
				{
					$db->query("DELETE FROM `buddyserv_jupedchans` WHERE `channel` = '".$channel."' LIMIT 1");
					self::$client->notice($nick, sprintf($user->language->CS_REMOVE_FORBIDEN, $channel));
					
					return $irc->send(sprintf(OPERWALL, self::$client->nick, sprintf($language->CS_UNFORBIDWALL, $channel, $nick)));
				}
			}
			
			else 
			{
				return self::$client->notice($nick, $dowhat." Invalid Syntax: FORBID <ADD|DEL> <channel> [reason]");
			}
	}
				
		
	/**
	 * Register a channel to the database. Registered
	 * channels have access to all chanserv related
	 * management functions.
	 * 
	 * @param string $nick
	 * @param string $arguments
	 * @return void
	 */
	static function register($nick, $arguments)
	{
		global $irc, $db, $me, $language;
		
		$user = user::getuser($nick);
		
		array_shift($arguments);
		$channel = array_shift($arguments);
		$description = mysql_real_escape_string(implode(' ', $arguments));
		
		if (!channel::exists($channel))
			return self::$client->notice($nick, sprintf($user->language->CS_NOSUCHCHANNEL, $channel));
			
		if (!$description)
			return self::$client->notice($nick, $user->language->CS_NEED_DESCRIPTION);
			
		$chan = channel::get($channel);
		
		if (!$user or !$user->identified() or (($user->level < SITE_ADMIN) and ($chan->can_reg != $nick)))
			return self::$client->notice($nick, sprintf($user->language->CS_REG_NOTALLOWED, $me->support_channel));
			
		if ($chan->active)
			return self::$client->notice($nick, sprintf($user->language->CS_ALREADY_REGISTERED, $channel));
			
		$query = $db->query("INSERT INTO `buddyserv_channels` (`name`, `founder`, `regdate`, `lastused`, `description`, `mlock`)
			VALUES ('".mysql_real_escape_string($channel)."', '{$nick}', '".current_time()."', '".current_time()."', '{$description}', 17408)");
		
		$chan->id = $db->insert_id($query);
		
		if ($chan->id == 0)
			return self::$client->notice($nick, sprintf($user->language->CS_REGISTERERROR, $channel));
			
		$chan->active = true;
		$chan->founder = $nick;
		
		$irc->send(sprintf(SJOIN, $me->name, time(), $chan->name, '+', self::$client->nick));
		self::$client->chanMode($chan->name, '+oq '.self::$client->nick.' '.$nick);
		
		$irc->send(sprintf(OPERWALL, self::$client->nick, sprintf($language->CS_REGISTERWALLOP, $channel, $nick, $description)));
		$irc->send(sprintf(NOTICE, self::$client->nick, $nick, sprintf($user->language->CS_NOW_REGISTERED, $channel)));
		
		return;
	}
	
	/**
	 * Command to allow a person to register the given channel.
	 * 
	 * @param string $nick
	 * @param string $channel
	 * @param string $founder
	 * @return void
	 */
	static function allow($nick, $channel, $founder)
	{
		global $irc, $db, $me;
		
		$user = user::getuser($nick);
		
		if ((!$user or !$user->identified() or ($user->level < SITE_ADMIN)))
			return chanserv::denied($nick);
			
		if (!$channel)
			return self::$client->notice($nick, "SYNTAX: ALLOW <channel> <founder>");
			
		if (!channel::exists($channel))
			return self::$client->notice($nick, sprintf($user->language->CS_NOSUCHCHANNEL, $channel));
			
		$chan = channel::get($channel);
		$target = user::sql_single($founder);
		
		if (!$target or ($target['enabled'] == 'no'))
			return self::$client->notice($nick, sprintf($user->language->NS_USERNOTREGISTERED, $founder));
			
		$chan->can_reg = $founder;
		return self::$client->notice($nick, sprintf($user->language->CS_CANREGISTER, $channel, $founder));
	}
	
	/**
	 * Modify a channels access list.
	 * 
	 * @param string $nick
	 * @param string $arguments
	 * @param string $channel
	 * @return void
	 */
	static function modaccess($nick, $arguments, $channel=false)
	{ 
		global $irc, $me, $db, $language;
		
		$type = array_shift($arguments);
		
		/* if this is a channel command, $channel will be used */
		if (!$channel)
			$channel = array_shift($arguments);
		
		$do = array_shift($arguments);
		$who = array_shift($arguments);
		$typebin = 0x0000;
		
		/* strip the periods from the commands */
		if (substr($type, 0, 1) == '.')
			$type = substr($type, 1, strlen($type));
			
		if (!$do)
			return self::$client->notice($nick, "Invalid Syntax.");
			
		if (strcasecmp($type, 'SOP') == 0)
			$typebin = chanserv::ACCESS_SOP;
			
		elseif (strcasecmp($type, 'AOP') == 0)
			$typebin = chanserv::ACCESS_AOP;
			
		elseif (strcasecmp($type, 'HOP') == 0)
			$typebin = chanserv::ACCESS_HOP;
			
		elseif (strcasecmp($type, 'VOP') == 0)
			$typebin = chanserv::ACCESS_VOP;
			
			
		else 
		{
			return chanserv::unknowncmd($nick, $type);
		}
		
		/* check for valid channel */
		$chan = channel::get($channel);
		$requester = user::getuser($nick);
		
		if (!$chan->active)
			return self::$client->notice($nick, sprintf($requester->language->CS_NOTREGISTERED, $channel));
			
		/* Check for proper access */
		if (stristr($do, 'LIST'))
		{
			if (($chan->has_access($nick, chanserv::ACCESS_AOP)) or ($chan->has_access($nick, chanserv::ACCESS_SOP)) or ($chan->founder == $nick))
			{
				$i=0;
				self::$client->notice($nick, sprintf($requester->language->CS_ACCESSLISTSTART, strtoupper($type), $chan->name));
				
				foreach ($chan->access as $who => $bin)
				{
					if (($bin & $typebin) > 0)
					{
						$i++;
						self::$client->notice($nick, "[{$i}] {$who}");
					}
				}
				
				return self::$client->notice($nick, $requester->language->CS_ACCESSLISTEND);
			}
			
			return chanserv::denied($nick);
				
		}
		
		if (stristr($do, 'ADD'))
		{
			if (!$who)
				return self::$client->notice($nick, "Syntax: ".strtoupper($type)." ADD <username>");
				
			$whoj = user::sql_single($who);
			
			if (!$whoj)
				return self::$client->notice($nick, $requester->language->CS_ACCESSONLYREG);
				
			if ($whoj['enabled'] == 'no')
				return self::$client->notice($nick, $requester->language->CS_ACCESSNOTDISABLED);
				
			/* Check if the issuer is allowed to peform this command */
			if (($chan->has_access($nick, chanserv::ACCESS_SOP)) or ($chan->founder == $nick) or ($requester->level > SITE_ADMIN ))
			{
				/* SOP by founder only */
				if ((($typebin & chanserv::ACCESS_SOP) > 0) and (($chan->founder != $nick)) and ($requester->level < SITE_SYSOP))
					return chanserv::denied($nick);
				
				/* the target alread has this access */	
				if ($chan->has_access($who, $typebin))
					return self::$client->notice($nick, sprintf($requester->language->CS_ACCESSEXISTS, $who, $channel));
				
				/* add the user to the channels access database */
				$chan->addaccess($who, $typebin);
				
				/* we store arrays in the database for access. as such we need to serialize the data */
				
				self::$client->notice('@'.$channel, sprintf($language->CS_ACCESSADDEDTELL, $nick, $who, strtoupper($type)));
				return self::$client->notice($nick, sprintf($requester->language->CS_ACCESSADDED, $who, $channel, strtoupper($type)));
			}
			
			return chanserv::denied($nick);
		}
		
		if (stristr($do, 'DEL'))
		{
			if (!$who)
				return self::$client->notice($nick, "Syntax: ".strtoupper($type)." DEL <username>");
		
			/* Get the targets user object */
			$whoj = user::getuser($who);
			
			if (($chan->has_access($nick, chanserv::ACCESS_SOP)) or ($chan->founder == $nick) or ($requester->level > 5 ))
			{
				if (($typebin & chanserv::ACCESS_SOP > 0) and ($chan->founder != $requester->nick) and ($requester->level < 7))
					return chanserv::denied($nick);
					
				if (!$chan->has_access($who, $typebin))
					return self::$client->notice($nick, sprintf($requester->language->CS_ACCESSDELNOFOUND, $who, $channel));
				
				/* remove the acccess in question */
				$chan->remaccess($who, $typebin);
				$serialize = serialize($chan->access);
				
				$db->query("UPDATE `buddyserv_channels` SET `accesslist` = '".$serialize."' WHERE `name` = '".$chan->name."' LIMIT 1");
					
				self::$client->notice('@'.$channel, sprintf($language->CS_ACCESSDELTELL, $nick, $who, strtoupper($type)));
				return self::$client->notice($nick, sprintf($requester->language->CS_ACCESSDEL, $who, $channel, strtoupper($type)));
			}
			
			return chanserv::denied($nick);
		}
	
	}
	
	/**
	 * If a user does not have access to a command, this is sent
	 * 
	 * @param string $nick
	 * @return void
	 */
	static function denied($nick)
	{
		$userObj = user::getuser($nick);
		
		return self::$client->notice($nick, $userObj->language->NS_ACCESSDENIED);
	}
		
 	/**
 	 * Sent when an unknown command is intercepted.
 	 * 
 	 * @param string $nick
 	 * @param string $command
 	 * @return void
 	 */
 	static function unknowncmd($nick,$command)
	{	
		$userObj = user::getuser($nick);
		
		return self::$client->notice($nick, sprintf($userObj->language->CS_UNKNOWNCOMMAND, "\2{$command}\2"));
	}
	
	/**
	 * Handle user joins to a channel. This only
	 * accepts one nickname per request.
	 * 
	 * @param user $nick
	 * @param channel $chan
	 * @param bit $binmode
	 * @return void
	 */
	static function handlejoin($nick, $chan, $binmode)
	{
		global $irc, $me, $chanmodes;
		
		$domode = '';
		$doparm = '';
		
		/* check if forbidden channel */
		if (($reason = $chan->is_juped()) != false)
		{
			$irc->send(sprintf(SJOIN, $me->name, time(), $chan->name, '+', chanserv::$client->nick));
			self::$client->chanMode($chan->name, '+ntmsOob '.chanserv::$client->nick.' *!*@'.$nick->vhost);
			//$irc->send(sprintf(MODE, $me->name, $chan->name, 'ntmsoqb '.chanserv::$client->nick.' '.chanserv::$client->nick.' *!*@'.$nick->host));

			if (($nick->level > SITE_MODERATOR) and ($nick->identified()))
			{
				self::$client->msg($chan->name, sprintf($nick->language->CS_ISJUPED, $chan->name));
			}
			
			else 
			{
				self::$partchans[$chan->name] = time() + 15;
				self::$client->kick($chan->name, $nick->nick, sprintf($nick->language->CS_CHANNELCLOSED, $reason));
				
				return;
			}
			
			self::$partchans[$chan->name] = time() + 15;
		}
				
			
		/* check if this is the first member */
		if ($chan->usercount() == 1)
		{
			$use_mode_persist = false;
			
			if ($chan->active)
			{
				if (($chan->name == $me->support_channel) or ($chan->name == $me->staff_channel) or ($chan->name == $me->user_channel))
				{
					$use_mode_persist = true;
					$irc->send(sprintf(SJOIN, $me->name, time(), $chan->name, '+', self::$client->nick.' '.$me->userbot['nick']));
					$irc->send(sprintf(MODE, self::$client->nick, $chan->name, '+vo '.$me->userbot['nick'].' '.self::$client->nick));
				}
				
				else 
				{
					$irc->send(sprintf(SJOIN, $me->name, time(), $chan->name, '+', self::$client->nick));
					$irc->send(sprintf(MODE, self::$client->nick, $chan->name, '+o '.self::$client->nick));
				}
	
				if (strlen($chan->topic) > 2)
					$irc->send(sprintf(TOPIC, self::$client->nick, $chan->name, $chan->topic));
					
				if ($chan->mlock)
				{
					foreach ($chanmodes as $mode => $bin)
					{
						if (($chan->mlock & $bin) > 0)
						{
							if (($bin & CMODE_KEY) > 0)
							{
								$domode .= '+'.$mode;
								$doparm .= $chan->chankey.' ';
							}
							
							else 
							{
								$domode .= '+'.$mode;
							}
						}
					}
					
					/* If its an OFFICIAL channel, we set the persist
					 * mode so the channel will always stay created instead
					 * of destructing after the last user has left. This is an IRCD
					 * feature. It doesnt change the way services acts in any way.
					 * 
					 * In the future, we cound check for this flag to make services
					 * remain in the channel, and skip re-setting modes/topics when
					 * it is set -- (make sure mode +z is ircd supported - lynk Dec09.09
					 */
					if ($use_mode_persist)
						$domode .= '+z';
						
					self::$client->chanMode($chan->name, $domode. ' '.$doparm);
					
					$domode = '';
					$doparm = '';
				}
							
			}
			
			/* unregistered channel */
			else 
			{
				/* New installs don't have any channels registered. If a user is
				 * joining the support or user channel, DONT kick them.
				 */
				if (($chan->name == $me->support_channel) or ($chan->name == $me->user_channel))
				{
					self::$client->join($chan->name);
				}
				
				elseif (!$nick->identified() or ($nick->level < SITE_ADMIN))
				{
					self::$partchans[$chan->name] = time() + 15;
					self::$client->join($chan->name);
					self::$client->chanMode($chan->name, '+ntmsob '.self::$client->nick.' *!*@*');
					self::$client->kick($chan->name, $nick->nick, sprintf($nick->language->CS_BADCHANNEL, $me->support_channel));
					return;
				}
			}
		}
		
		if ($chan->active)
		{
			/* not identified person */
			if (($nick->modes & UMODE_REGISTERED) == 0)
			{
				if ($chan->restricted)
				{
					self::$client->chanMode($chan->name, '+b *!*@'.(($nick->vhost) ? $nick->vhost : $nick->host));
					return self::$client->kick($chan->name, $nick->nick, $nick->language->CS_NOPERMISSIONHERE);	
				}
				
				if (($binmode & CMODE_FOUNDER) > 0)
				{
					$domode .= '-q';
					$doparm .= $nick->nick.' ';
				}
	
				if (($binmode & CMODE_PROTECT) > 0)
				{
					$domode .= '-a';
					$doparm .= $nick->nick.' ';
				}
	
				if (($binmode & CMODE_OP) > 0)
				{
					$domode .= '-o';
					$doparm .= $nick->nick.' ';
				}
	
				if (($binmode & CMODE_HALFOP) > 0)
				{
					$domode .= '-h';
					$doparm .= $nick->nick.' ';
				}
	
				if (strlen($domode) > 0)
					self::$client->chanMode($chan->name, $domode .' '. $doparm);
	
				if ($akick = $chan->matchingAkick($nick))
				{
					self::$client->chanMode($chan->name, '+b '.$akick['mask']);
					self::$client->kick($chan->name, $nick->nick, 'Banned: '.$akick['reason']);
					return;	
				}
			}
			
			else 
			{			
				if (($chan->name == $me->support_channel) and ($nick->level < SITE_MODERATOR))
				{
					$enabled = (($nick->forbid == 'yes') ? "\0034,0disabled\003" : "\0033,0enabled\003");
	
					$infostring = $enabled." \00312".$me->url."/userdetails.php?id=". $nick->id ."\003";
					$irc->send(sprintf(MSG, $me->userbot['nick'], $chan->name, "\2[\2user ({$nick->nick})\2]\2 {$infostring}"));
				}
				
				if ($nick->level > SITE_ADMIN)
				{
					$domode .= '+qo';
					$doparm .= $nick->nick. ' '. $nick->nick.' ';
				}
				
				elseif ($nick->level == SITE_ADMIN)
				{
					$domode .= '+ao';
					$doparm .= $nick->nick. ' '. $nick->nick.' ';
				}
				
				elseif ($chan->founder == $nick->nick)
				{
					$domode .= '+qo';
					$doparm .= $nick->nick. ' '. $nick->nick.' ';
				}
				
				/**
				 * I changed the has_access to perform recurively. You
				 * must check the access in descending order, sop, aop. hop,
				 * etc. Luckily this is how I had already coded it, so no changes
				 * were made. We will fix this in the version2.0 branch. --lynk
				 */
				elseif ($chan->has_access($nick->nick, chanserv::ACCESS_SOP))
				{
					$domode .= '+ao';
					$doparm .= $nick->nick. ' '. $nick->nick. ' ';
				}
				
				elseif ($chan->has_access($nick->nick, chanserv::ACCESS_AOP))
				{
					$domode .= '+o';
					$doparm .= $nick->nick. ' ';
				}
				
				elseif ($chan->has_access($nick->nick, chanserv::ACCESS_HOP))
				{
					$domode .= '+h';
					$doparm .= $nick->nick. ' ';
				}
				
				elseif ($chan->has_access($nick->nick, chanserv::ACCESS_VOP))
				{
					$domode .= '+v';
					$doparm .= $nick->nick. ' ';
				}
				
				elseif ($akick = $chan->matchingAkick($nick))
				{
					self::$client->chanMode($chan->name, '+b '.$akick['mask']);
					self::$client->kick($chan->name, $nick->nick, 'Banned: '.$akick['reason']);
					return;	
				}
				
				elseif ($chan->restricted)
				{
					self::$client->chanMode($chan->name, '+b *!*@'. (($nick->vhost) ? $nick->vhost : $nick->host));
					self::$client->kick($chan->name, $nick->nick, $nick->language->CS_NOPERMISSIONHERE);
					return;	
				}
				
				else 
				{
					
					if (($chan->name == $me->support_channel) or ($chan->name == $me->staff_channel) or ($chan->name == $me->user_channel))
					{
						if ($nick->level == SITE_UPLOADER)
						{
							$domode .= '+v';
							$doparm .= $nick->nick;
						}
						
						elseif ($nick->level == SITE_MODERATOR)
						{
							$domode .= '+o';
							$doparm .= $nick->nick;
						}
						
						elseif ($nick->level == SITE_ADMIN)
						{
							$domode .= '+oa';
							$doparm .= $nick->nick. ' '.$nick->nick;
						}
						
						elseif ($nick->level > SITE_ADMIN)
						{
							$domode .= '+oq';
							$doparm .= $nick->nick. ' '.$nick->nick;
						}
					} /* end offical channels */
					
	
				}
				
				if (strlen($domode) > 0)
					self::$client->chanMode($chan->name, $domode.' '.$doparm);
			}
				
		}
		
		/* Grabs users modes set it convert to binary */
		$each = str_split($domode);
		$bin = 0x00000;
		
		foreach ($each as $null => $current)
		{
			if (($current == '+') or ($current == '-'))
				continue;
				
			if (array_key_exists($current, $chanmodes))
				$bin = ($bin | $chanmodes[$current]);
			
		}
	
		$chan->adduser($nick->nick, $bin);
		$nick->addchannel($chan->name, $bin);
	
	}
	
	/**
	 * Just a debug function to return true
	 * binary mode from a string
	 * 
	 * @param string $nick
	 * @param string $modes
	 * @return void
	 */
	static function binarymode($nick, $modes)
	{
		global $chanmodes;
		
		$bin = 0x000000;
		
		$each = str_split($modes);
		
		foreach ($each as $null => $current)
		{
			if (($current == '+') or ($current == '-'))
				continue;
				
			if (array_key_exists($current, $chanmodes))
				$bin = ($bin | $chanmodes[$current]);
			
		}
		$array = array('lynk'	=> chanserv::ACCESS_SOP);
		$array = serialize($array);
		self::$client->notice($nick, 'Bin: '. $bin .' modes: '. $modes .'-- '.$array);
	}
	
}


?>