<?php

/**
 * BuddyServ IRC Services
 * 
 * Ignore object handler.
 * 
 * @name Ignore
 * @author Raymond Lynk <me@raymondj.ca>
 * @copyright (C) 2010, Raymond Lynk
 * @link http://buddyserv.raymondj.ca
 * @package Core
 * @version $Id: Class.ignore.php 350 2009-12-03 07:55:58Z lynk $
 */

/*
 * 		 ______           __     __         _______                        
 * 		|   __ \.--.--.--|  |.--|  |.--.--.|     __|.-----.----.--.--.     
 * 		|   __ <|  |  |  _  ||  _  ||  |  ||__     ||  -__|   _|  |  |     
 * 		|______/|_____|_____||_____||___  ||_______||_____|__|  \___/      
 *  				    			   |_|                                 
 *                                                                         
 */
class ignore 
{
	public $id;
	public $mask;
	public $added;
	public $expires;
	public $oper;
	public $reason;
	public $regex;
	
	public static $list = array();
	
	/**
	 * Populate the list of ignore entries from the database
	 * 
	 * @return void
	 */
	public static function populate()
	{
		global $db;
		
		$query = $db->query("SELECT * FROM `buddyserv_ignore`");
		
		/* no results? makes it easy. */
		if ($db->num_rows($query) == 0)
			return;
			
		while ($each = $db->fetch_array($query))
		{
			/* if its an expired ignore, dont bother continuing */
			if ($each['expires'] < time())
				continue;
				
			$instance = new ignore();
			
			/* Cycle each row value and add it to the object. */
			foreach ($each as $name => $value)
			{
				$instance->{$name} = $value;
			}
			
			$instance->regex = make_wild($instance->mask);
			
			/* Add it to our list */
			ignore::$list[strtolower($instance->mask)] = $instance;
		}
	}
	
	/**
	 * Gets an instance of ignore matching
	 * the given mask. Returns false if not found.
	 * 
	 * @param string $mask
	 * @return bool|ignore
	 */
	public static function get($mask)
	{
		if (!self::exists($mask))
			return false;
			
		$object = &self::$list[strtolower($mask)];
		return $object;
	}
	
	/**
	 * Removes the current ignore from the database,
	 * and unsets it from the list. Returns true/false.
	 * 
	 * @return bool
	 */
	public function delete()
	{
		global $db;
		
		if (!$this->id)
			return false;
			
		/* delete from database */
		$db->query(sprintf("DELETE FROM `buddyserv_ignore` WHERE `id` = %d", $this->id));
		unset(ignore::$list[strtolower($this->mask)]);
		
		return true;
	}
	
	/**
	 * Save the current ignore instance to the database
	 * and sets the id of the new entry. Also adds it to the
	 * list of active ignores.
	 * 
	 * @return bool
	 */
	public function save()
	{
		global $db;
		
		if (!(($this->mask) or ($this->oper) or ($this->reason)))
			return false;
			
		$query = $db->query(sprintf("INSERT INTO `buddyserv_ignore` (`added`, `expires`, `mask`, `oper`, `reason`) VALUES (%d, %d, '%s', '%s', '%s')",
			time(),
			$this->expires,
			mysql_real_escape_string($this->mask),
			mysql_real_escape_string($this->oper),
			mysql_real_escape_string($this->reason)
		));
		
		if ($db->num_affected($query) == 0)
			return false;
			
		$this->id = $db->insert_id($query);
		ignore::$list[strtolower($this->mask)] = $this;
		return true;
	}
	
	/**
	 * Check if a given mask is an ignore instance.
	 * This is NOT for checking if someone is ignored.
	 * 
	 * @param string $mask
	 * @return void
	 */
	public static function exists($mask)
	{
		$mask = strtolower($mask);
		
		if (array_key_exists($mask, self::$list))
			return true;
			
		return false;
		
	}
	
	/**
	 * Check if the given mask matches an ignore
	 * entry. If it does, returns true, else false.
	 * 
	 * @param string $mask
	 * @return void
	 */
	public static function is($mask)
	{
		if (count(self::$list) == 0)
			return false;
			
		foreach (self::$list as $eachmask => $instance)
		{
			if (preg_match('/^'.$instance->regex.'$/i', $mask))
				return true;
		}
		
		return false;
	}
	
	/**
	 * Checks the ignores, and deletes the
	 * ones that have expired.
	 * 
	 * @return void
	 */
	public static function runExpire()
	{
		global $db, $language;
		
		/* if we have no ignores, we have nothing to expire. */
		if (count(self::$list) == 0)
			return;
		
		$list = array();
		
		foreach (self::$list as $null => $instance)
		{
			if ($instance->expires < time())
			{
				operserv::$client->operwall(sprintf($language->OS_IGNORE_EXPIRE, $instance->mask));
				$db->query(sprintf("DELETE FROM `buddyserv_ignore` WHERE `id` = %d", $instance->id));
				unset(self::$list[$instance->mask]);
			}
		}
	}
}

?>