<?php

/**
 * BuddyServ IRC Services
 * 
 * Monitor IRC pseudo client handler. Manages
 * extra service functions, and graphing commands.
 * 
 * @name Monitor
 * @author Raymond Lynk <me@raymondj.ca>
 * @copyright (C) 2010, Raymond Lynk
 * @link http://buddyserv.raymondj.ca
 * @package Clients
 * @version $Id: Class.monitor.php 359 2009-12-10 11:11:38Z lynk $
 */

/*
 * 		 ______           __     __         _______                        
 * 		|   __ \.--.--.--|  |.--|  |.--.--.|     __|.-----.----.--.--.     
 * 		|   __ <|  |  |  _  ||  _  ||  |  ||__     ||  -__|   _|  |  |     
 * 		|______/|_____|_____||_____||___  ||_______||_____|__|  \___/      
 *  				    			   |_|                                 
 *                                                                         
 */

class monitor
{
	static $client;
	static $last_read =0;
	static $enabled=true;
	static $last_rrd=0;
	
	/**
	 * Initialize the current service client
	 * 
	 * @param string $nick
	 * @param string $username
	 * @param string $host
	 * @param string $name
	 * @return void
	 */
	static function initialize($nick, $username, $host, $name, $modes)
	{
		self::$last_read = current_time();
		self::$client = new serviceclient();
		self::$client->initialize($nick, $username, $host, $name, $modes);
	}
	
	/**
	 * magic get.
	 * 
	 * @param string $what
	 * @return mixed
	 */
	function __get($what)
	{
		if (self::$client->$what)
			return self::$client->$what;
			
	}
	
	/**
	 * Introduce the client to the network.
	 * 
	 * @return void
	 */
	static function introduce()
	{
		self::$client->introduce();
	}
	
	/**
	 * Check if its time to update the rrd graph, and
	 * create an image for the most recent numbers.
	 * 
	 * @return void
	 */
	static function check_rrd()
	{
		global $me, $irc, $site, $config;
		
		if ($site)
			$rrd_graph_dir = $config['rrd']['development'];
			
		else
			$rrd_graph_dir = $config['rrd']['production'];
			
		if ($me->use_rrd)
		{
			/* check if its time to run the update */
			if ((self::$last_rrd == 0) or ((current_time() - self::$last_rrd) > 300))
			{
				$count = user::count_online();
				$guest = user::count_guests();

				@exec('rrdtool update '.$me->rrd_file.' N:'.$count.':'.$guest);
				
				/* Graph the current irc users first */
				rrd::$title = 'IRC Graphs';
				rrd::$vertical = 'User Count';
				
				rrd::update(array($count, $guest));
				rrd::graph($rrd_graph_dir.'hour.png', '-3600');
				rrd::graph($rrd_graph_dir.'day.png', '-86400');
				rrd::graph($rrd_graph_dir.'week.png', '-604800');
				rrd::graph($rrd_graph_dir.'month.png', '-2419200');
				rrd::graph($rrd_graph_dir.'year.png', '-31536000');
				
				/* Generate Lines Graph */
				rrd::$rrdfile = 'rrd/irclines.rrd';
				rrd::$title = 'IRC Send/Recv';
				rrd::$vertical = 'Send/Receive Lines';
				
				rrd::update(array(($irc->recvLines + $irc->sendLines), $irc->recvLines, $irc->sendLines));
				
				$irc->recvLines = 0;
				$irc->sendLines = 0;
				
				rrd::graphLines($rrd_graph_dir.'hourlines.png', '-3600');
				rrd::graphLines($rrd_graph_dir.'daylines.png', '-86400');
				rrd::graphLines($rrd_graph_dir.'weeklines.png', '-604800');
				rrd::graphLines($rrd_graph_dir.'monthlines.png', '-2419200');
				rrd::graphLines($rrd_graph_dir.'yearlines.png', '-31536000');

				self::$last_rrd = current_time();
			}
		}
	}
}

?>