<?php

/**
 * BuddyServ IRC Services
 * 
 * NickServ account identification services
 * 
 * @name NickServ
 * @author Raymond Lynk <me@raymondj.ca>
 * @copyright (C) 2010, Raymond Lynk
 * @link http://buddyserv.raymondj.ca
 * @package Clients
 * @version $Id: Class.nickserv.php 355 2009-12-10 07:47:20Z lynk $
 */

/*
 * 		 ______           __     __         _______                        
 * 		|   __ \.--.--.--|  |.--|  |.--.--.|     __|.-----.----.--.--.     
 * 		|   __ <|  |  |  _  ||  _  ||  |  ||__     ||  -__|   _|  |  |     
 * 		|______/|_____|_____||_____||___  ||_______||_____|__|  \___/      
 *  				    			   |_|                                 
 *                                                                         
 */

class nickserv
{
	static $client;
	
	/**
	 * Initialize this service client.
	 * 
	 * @param string $nick
	 * @param string $username
	 * @param string $host
	 * @param string $name
	 * @return void
	 */
	static function initialize($nick, $username, $host, $name, $modes)
	{
		self::$client = new serviceclient();
		self::$client->initialize($nick, $username, $host, $name, $modes);
	}
	
	/**
	 * Magic getter
	 * 
	 * @param string $what
	 * @return mixed
	 */
	function __get($what)
	{
		if (self::$client->$what)
			return self::$client->$what;
			
	}
	
	/**
	 * Introduce this client to the uplink server.
	 * 
	 * @return void
	 */
	static function introduce()
	{
		self::$client->introduce();
	}
	
	/**
	 * Called to parse any messages to NickServ
	 *
	 * @param string $nick
	 * @param string $arguments
	 * @return void
	 */
	static function parser($nick, $arguments)
	{
		if (!strcasecmp($arguments[0], 'ENABLE'))
			return nickserv::enable($nick, $arguments[1]);
		
		if (!strcasecmp($arguments[0], 'IDENTIFY'))
			return nickserv::identify($nick, $arguments[1]);
	
		if (!strcasecmp($arguments[0], 'VHOST'))
			return nickserv::vhost($nick, $arguments);
			
		if (!strcasecmp($arguments[0], 'LOGIN'))
			return nickserv::login($nick, $arguments[1], $arguments[2]);
			
		if (!strcasecmp($arguments[0], 'INFO'))
			return nickserv::info($nick, $arguments[1]);
			
		if (!strcasecmp($arguments[0], 'GHOST'))
			return nickserv::ghost($nick, $arguments[1], $arguments[2]);
			
		if (!strcasecmp($arguments[0], 'RECOVER'))
			return nickserv::recover($nick, $arguments[1], $arguments[2]);
			
		if (!strcasecmp($arguments[0], 'DISABLE'))
			return nickserv::disable($nick, $arguments);
			
		if (!strcasecmp($arguments[0], 'LANGUAGE'))
			return nickserv::language($nick, $arguments[1], (isset($arguments[2]) ? $arguments[2] : false));
		
		if (!strcasecmp($arguments[0], 'HELP'))
		{
			array_shift($arguments);
			
			if (count($arguments) == 0)
				return nickserv::$client->help($nick, 'nickserv');
			
			return nickserv::$client->help($nick, 'nickserv', $arguments);
		}
		
		return nickserv::unknowncmd($nick, $arguments[0]);
	}
	
	/**
	 * Command to change the language that a user
	 * is spoken to with services.
	 * 
	 * @param string $nick
	 * @param string $command
	 * @param string $lang
	 * @return void
	 */
	public static function language($nick, $command, $lang)
	{
		global $language;
		
		$userObj = user::getuser($nick);
		
		if (($userObj->modes & UMODE_REGISTERED) == 0)
			return self::$client->notice($nick, sprintf($userObj->language->CS_NOTIDENTIFIED, self::$client->nick));
			
		if (!strcasecmp($command, 'SET'))
		{
			if (!$lang)
				return self::$client->notice($nick, 'SYNTAX: LANGUAGE SET <language>');
				
			if (!language::exists($lang))
				return self::$client->notice($nick, sprintf($userObj->language->NS_LANGUAGE_UNKNOWN, $lang));
				
			/* tell the user object to set the language. We use
			 * an actual language object as the passed item, just to
			 * ensure that we are using a valid language.
			 */
			$userObj->setLanguage(new language($lang));
			
			return self::$client->notice($nick, sprintf($userObj->language->NS_LANGUAGE_SET, strtoupper($lang)));
			
		}
			
		elseif (!strcasecmp($command, 'LIST'))
		{
			self::$client->notice($nick, sprintf($userObj->language->NS_LANGUAGE_LIST, self::$client->nick));
			
			$i = 0;
			
			foreach (language::$available as $name => $array)
			{
				self::$client->notice($nick, '['.++$i.'] '.strtoupper($name));
					
			}
			
			return;
		}
		
		return self::$client->notice($nick, sprintf($userObj->language->CS_UNKNOWNCOMMAND, 'LANGUAGE '.$command));
	}
	
	/**
	 * Command not known response
	 *
	 * @param string $nick
	 * @param string $command
	 * @return null
	 */
	static function unknowncmd($nick,$command)
	{	
		$userObj = user::getuser($nick);
		
		return self::$client->notice($nick, sprintf($userObj->language->CS_UNKNOWNCOMMAND, "\2{$command}\2"));
	}
	
	/**
	 * Enables a username that is currently disabled.
	 * 
	 * @param string $nick
	 * @param string $target
	 * @return void
	 */
	static function enable($nick,$target)
	{
		global $irc, $me, $language;
		$user = user::getuser($nick);
		
		if (($user->modes & UMODE_REGISTERED) == 0 or ($user->level < SITE_MODERATOR))
			return nickserv::denied($nick);
		
		if (!$target)
			return self::$client->notice($nick, "SYNTAX: Enable <username>");
			
		$offender = user::sql_single($target);
	
		if (!$offender)
			return self::$client->notice($nick, sprintf($user->language->NS_USERNOTREGISTERED, $target));
			
		if ($offender['class'] >= $user->level)
		{
			self::$client->notice($nick, $user->language->NS_ENABLEDISABLEEQUAL);
			$irc->send(sprintf(OPERWALL, self::$client->nick, sprintf($language->NS_FAILENABLEDISABLE, $user->nick, 'ENABLE', "\00312".$offender['username']."\003")));
			return;
		}
			
		if ($offender['enabled'] == 'yes')
			return self::$client->notice($nick, sprintf($user->language->NS_NOTDISABLED, $target));
			
		user::enable($offender['username'], 'Enabled by '.$nick);
		
		$irctarget = user::getuser($target);
		
		/* if this person is online and identified, say bye */
		if (($irctarget instanceof user))
		{
			$irctarget->forbid = false;
			self::$client->notice($irctarget->nick, $irctarget->language->NS_REACTIVATED);
		}
		
		self::$client->notice($nick, sprintf($user->language->NS_ENABLEDNOW, $target));
		return $irc->send(sprintf(OPERWALL, self::$client->nick, sprintf($language->NS_ENABLETELL, $nick, $target)));
	}
	
	/**
	 * Disable a currently active username.
	 * 
	 * @param string $nick
	 * @param array $arguments
	 * @return void
	 */
	static function disable($nick,$arguments)
	{
		global $irc, $me, $language;
		$user = user::getuser($nick);
		
		if (($user->modes & UMODE_REGISTERED) == 0 or ($user->level < SITE_MODERATOR))
			return self::denied($nick);
			
		array_shift($arguments);
		$target = array_shift($arguments);
		$reason = implode(' ', $arguments);
		
		if (!$target)
			return self::$client->notice($nick, "SYNTAX: Disable <username> <reason>");
			
		$offender = user::sql_single($target);
	
		if (!$offender)
			return self::$client->notice($nick, sprintf($user->language->NS_USERNOTREGISTERED, $target));
			
		if ($offender['class'] >= $user->level)
		{
			self::$client->notice($nick, $user->language->NS_ENABLEDISABLEEQUAL);
			$irc->send(sprintf(OPERWALL, self::$client->nick, sprintf($language->NS_FAILENABLEDISABLE, $user->nick, 'DISABLE', $offender['username'])));
			return;
		}
			
		if ($offender['enabled'] == 'no')
			return self::$client->notice($nick, sprintf($user->language->NS_ALREADYDISABLED, $target));
			
		user::disable($nick, $offender['username'], $reason);
		
		$irctarget = user::getuser($target);
		
		/* if this person is online and identified, say bye */
		if (($irctarget instanceof user) && ($irctarget->identified()))
		{
			self::$client->notice($irctarget->nick, sprintf($irctarget->language->NS_DISABLEDNOW, $target, $reason));
			$irc->send(sprintf(KILL, 
			$me->nickserv['nick'], 
			$target, 
			$me->name, 
			$me->nickserv['host'], 
			$me->nickserv['username'],
			$me->nickserv['nick'],
			sprintf($irctarget->language->NS_DISABLEDNOW, $target, $reason)));
			
			user::remuser($irctarget);
		}
		
		self::$client->notice($nick, sprintf($user->language->NS_DISABLEDNOW, $target, $reason));
		return $irc->send(sprintf(OPERWALL, self::$client->nick, sprintf($language->NS_DISABLETELL, $nick, $target, $reason)));
	}
	
	/**
	 * Sets a virtual host for a user
	 * 
	 * @param string $nick
	 * @param array $arguments
	 * @return void
	 */
	static function vhost($nick, $arguments)
	{
		global $irc, $me, $db, $language;
		$user = user::getuser($nick);
		array_shift($arguments);
		$type = array_shift($arguments);
		$target = array_shift($arguments);
		$host = false;
		
		if (count($arguments) > 0)
			$host = array_shift($arguments);
		
		if (!$user->identified())
			return self::denied($nick);
			
		if (!strcasecmp($type, 'REQUEST'))
		{
			if (!$target)
				return self::$client->notice($nick, 'SYNTAX: REQUEST <vhost>');
				
			/* check invalid characters */
			if ((!preg_match('/^[A-Za-z0-9\.-]+$/', $target)))
			{
				self::$client->notice($nick, $user->language->NS_BREAKME);
			}
			
			$vhost = new vhost();
			$vhost->owner = $user->id;
			$vhost->host = $target;
			$vhost->username = strtolower($nick);
			
			/* add it to the requests */
			vhost::addRequest($vhost);
			
			self::$client->notice($nick, sprintf($user->language->NS_VHOST_REQUESTED, $target));
			self::$client->operwall(sprintf($language->NS_VHOST_REQUESTEDTELL, $nick, $target));
			
			return;
		}
		
		/* user has insufficant access, or is not identified. */
		if ((!$user->identified()) or ($user->level < SITE_MODERATOR))
		{
			return self::denied($nick);
		}
		
		if (!strcasecmp($type, 'DENY'))
		{
			if (!$target)
				return self::$client->notice($nick, 'SYNTAX: VHOST DENY <username>');
				
			$waiting = vhost::getRequests($target);
			
			if (!($waiting instanceof vhost))
				return self::$client->notice($nick, sprintf($user->language->NS_VHOSTNOTEXIST, $target));
			
			vhost::removeRequest($target);
			self::$client->notice($nick, sprintf($user->language->NS_VHOST_DENIED, $target));
			
			$isOnline = user::getuser($target);
			
			if (!($isOnline instanceof user))
				return;
				
			self::$client->notice($target, sprintf($isOnline->language->NS_VHOST_DENIED, $waiting->host));
			return;
		}
		
		if (!strcasecmp($type, 'ACCEPT'))
		{
			if (!$target)
				return self::$client->notice($nick, 'SYNTAX: VHOST ACCEPT <username>');
				
			$waiting = vhost::getRequests($target);
			
			if (!($waiting instanceof vhost))
				return self::$client->notice($nick, sprintf($user->language->NS_VHOSTNOTEXIST, $target));
			
			/* Delete the old vhost if it does exist */
			$oldVhost = new vhost();
			$oldVhost->owner = $waiting->owner;
			$oldVhost->deleteByUser();
			
			/* Set the new vhost. */
			$waiting->setby = $nick;
			$waiting->added = time();
			
			if (!$waiting->save())
				return self::$client->notice($nick, $user->language->MS_INTERNALERROR);
				
			self::$client->notice($nick, sprintf($user->language->NS_VHOST_SET, $target, $waiting->host));
			$irc->send(sprintf(OPERWALL, self::$client->nick, sprintf($language->NS_VHOSTTELL, $nick, $target, $waiting->host)));
			
			vhost::removeRequest($target);
			
			/* check if target is online and identified. set hostname */
			$is_online = user::getuser($target);
			
			if (($is_online) and ($is_online->identified()))
			{
				$irc->send(sprintf(RAW, ':'.$me->name.' ENCAP * SVSMODE '.$nick.' '.$object->ts .' +x'));
				$irc->send(sprintf(RAW, ':'.$me->name.' ENCAP * CHGHOST '.$target.' :'.$waiting->host));
					
				$is_online->vhost = $waiting->host;
				return self::$client->notice($target, sprintf($is_online->language->NS_VHOSTISNOW, $waiting->host));
			}
			
			return;
			
		}
		
		if (!strcasecmp($type, 'WAITING'))
		{
			$waiting = vhost::getRequests();
			
			if (!is_array($waiting))
				return self::$client->notice($nick, $user->language->NS_NOVIRTUALHOSTS);
				
			self::$client->notice($nick, sprintf($user->language->NS_VHOST_WAITING, count($waiting), self::$client->nick));
			
			foreach ($waiting as $username => $vhost)
			{
				self::$client->notice($nick, '['.$username.'] '.$vhost->host);
			}
			
			self::$client->notice($nick, $user->language->NS_VHOST_END);
			
			return;
		}
		
		if (strcasecmp($type, 'LIST') == 0)
		{
			$query = $db->query("SELECT vhost.*,user.username FROM `buddyserv_vhosts` vhost JOIN `users` user ON user.id=vhost.userid");
			
			if ($db->num_rows($query) == 0)
				return self::$client->notice($nick, $user->language->NS_NOVIRTUALHOSTS);

			$i = 1;
			self::$client->notice($nick, $user->language->NS_VHOST_START);
			
			while ($array = $db->fetch_array($query))
			{
				self::$client->notice($nick, $i.') '.$array['username'].' ['.$array['vhost'].']');
			}
			
			return self::$client->notice($nick, $user->language->NS_VHOST_END);
		}
		
		if (strcasecmp($type, 'ADD') == 0)
		{
			if (!$host)
				return self::$client->notice($nick, "SYNTAX: VHOST <add|list|del> <username> <vhost>");
			
			/* Dont let people use my special vhost */
				if (strcasecmp($host, 'buddyserv.author') == 0)
					return self::$client->notice($nick, sprintf($user->language->NS_VHOST_NOTALLOWED, 'buddyserv.author'));
					
			/* wtf ? dont let people break shit  by using funky shit in vhosts */
			if ((!preg_match('/^[A-Za-z0-9\.-]+$/', $host)) and ($host))
			{
				self::$client->notice($nick, $user->language->NS_BREAKME);
				$irc->send(sprintf(OPERWALL, self::$client->nick, sprintf($language->NS_BREAKMETELL, $nick, $target, $host)));
				return;
			}
			
			/* Get an array of user information */
			$array = user::sql_single($target);
			$vhost = new vhost();
			
			/* not an array, not registered person */
			if (!$array)
				return self::$client->notice($nick, sprintf($user->language->NS_USERNOTREGISTERED, $target));
					
			$vhost->findByUser($target);
	
			$vhost->owner = $array['id'];
			$vhost->setby = $nick;	
			$vhost->host = $host;
			
			/* Check for successfull save */
			if ($vhost->save() == true)
			{
				self::$client->notice($nick, sprintf($user->language->NS_VHOST_SET, $target, $host));
				$irc->send(sprintf(OPERWALL, self::$client->nick, sprintf($language->NS_VHOSTTELL, $nick, $target, $host)));
			
			/* check if target is online and identified. set hostname */
			
				$is_online = user::getuser($target);
				
				if (($is_online) and ($is_online->identified()))
				{
					$irc->send(sprintf(RAW, ':'.$me->name.' ENCAP * SVSMODE '.$nick.' '.$object->ts .' +x'));
					$irc->send(sprintf(RAW, ':'.$me->name.' ENCAP * CHGHOST '.$target.' :'.$host));
					
					$is_online->vhost = $host;
					return self::$client->notice($target, sprintf($is_online->language->NS_VHOSTISNOW, $host));
				}
			}
			
			/* failed save. */
			else
			{
				return self::$client->notice($nick, "An internal error occured trying to save that virtual host.");
			}
			
		}
		
		if (strcasecmp($type, 'DEL') == 0)
		{
			$array = user::sql_single($target);
			$vhost = new vhost();
			
			$vhost->owner = $array['id'];
			
			if ($vhost->deleteByUser() == true)
			{	
				return self::$client->notice($nick, sprintf($user->language->NS_VHOSTRESET, $target));
			}
			
			else
				return self::$client->notice($nick, sprintf($user->language->NS_VHOSTNOTEXIST, $target));
		}
		
		/* unknown type */
		return self::$client->notice($nick, sprintf($user->language->CS_UNKNOWNCOMMAND, "VHOST ".$type));
	}
	
	/**
	 * Info command uses to show user information
	 * 
	 * @param string $nick
	 * @param string $target
	 * @return void
	 */
	static function info($nick,$target)
	{
		global $irc, $me;
		
		$client = user::getuser($nick);
		
		if (!$target)
			return self::$client->notice($nick, "SYNTAX: INFO <username>");
			
		$who = user::sql_single($target);
		
		if ((($client->modes & UMODE_REGISTERED) > 0) and ($client->level > SITE_UPLOADER))
			$specials = true;
		
		else 
			$specials = false;
		
		if (!$who)
			return self::$client->notice($nick, sprintf($client->language->NS_USERNOTREGISTERED, $target));
			
			
			if ((!$specials) and $who['enabled'] == 'no')
				return self::$client->notice($nick, $client->language->NS_INFO_DISABLED);
				
			$lines = array();
			$check_online = user::getuser($target);
	
			if ($who['enabled'] == 'no')
					$lines[] = $client->language->NS_INFO_DISABLED;
				
			if (!$check_online)
			{
					$lines[] = sprintf($client->language->CS_NOUSERONIRC, $target);
					
			}
			
			else
			{ 
				$lines[] = sprintf($client->language->NS_INFO_START, $target, $check_online->name);
	
				if (!$check_online->identified())
					$lines[] = sprintf($client->language->NS_INFO_NOTIDENT, nickserv::$client->nick);
							
				if (!$specials)
					$lines[] = sprintf($client->language->NS_INFO_ONIRCFROM, $check_online->username.'@'.$check_online->vhost);
				
				else
					$lines[] = sprintf($client->language->NS_INFO_ONIRCFROM, $check_online->username.'@'.$check_online->host);
			}
			
				if ($who['donor'] == 'yes')
					$lines[] = $client->language->NS_INFO_DONOR;
			
			$level = $who['class'];
			
			if ($level)
					$lines[] = sprintf($client->language->NS_INFO_CLASS, strtoupper(user::$levels[$level]));
			
					$lines[] = sprintf($client->language->NS_INFO_TIMEREGISTERED, strftime("%c", $who['added']));
			
			if ($specials)
			{
				$level  = $who['class'];
					$lines[] = sprintf($client->language->NS_INFO_SITEIP, ($who['ip'] ? $who['ip'] : 'unknown'));
					$lines[] = sprintf($client->language->NS_INFO_EMAIL, $who['email']);
					$lines[] = sprintf($client->language->NS_INFO_PROFILEURL, "\00312".$me->url."/userdetails.php?id=".$who['id']."\003");
					
	 
			//	if ($who['warned'] == 'yes')
			//		$lines[] = "      \0034This user has an active warning\003";
					
				if ($who['modcomment'])
				{
					$msg =  explode("\n", str_replace("\r", '', $who['modcomment']));
					
					if (count($msg))
					$lines[] = sprintf($client->language->NS_INFO_MODCOMMENT, array_shift($msg));
				}
			}
			
			foreach ($lines as $message)
			{
				self::$client->notice($nick, $message);	
			}	
			
			self::$client->notice($nick, $client->language->NS_INFO_END);
			
	}
	
	/**
	 * Function to send 'access denied' string
	 *
	 * @param string $nick
	 */
	static function denied($nick)
	{	
		$userObj = user::getuser($nick);
		
		self::$client->notice($nick, $userObj->language->NS_ACCESSDENIED);
	}
	
	/**
	 * Recovers the target nickname from a user
	 * who should not be using it.
	 * 
	 * @param string $nick
	 * @param string $target
	 * @param string $password
	 * @return void
	 */
	static function recover($nick, $target, $password)
	{
		global $irc, $me;
		
		if (!$target or !$password)
			return self::$client->notice($nick, "Syntax: RECOVER <nick> <password>");
			
		$user = user::getuser($target);
		$first = user::getuser($nick);
		
		if (!$user)
			return self::$client->notice($nick, sprintf($first->language->CS_NOUSERONIRC, $target));
			
		if ($nick == $target)
			return self::$client->notice($nick, sprintf($first->language->NS_NOSELF, 'RECOVER'));
			
		if ($user->password != (md5($user->secret . $password . $user->secret)))	
			return self::denied($nick);
			
		/* passed all security */
		
		$time = time();
		$guest = 'Guest' . substr($time,(strlen($time) - 4)) .rand(0,100); 
		
	//	$irc->send(sprintf(RESV, $me->nickserv['nick'], $target, 'Registered: /msg '.$me->nickserv['nick'].' RELEASE '.$target.' <password> to reclaim'));
		$irc->send(sprintf(SVSNICK, $target, $user->ts, $guest, time()));
		self::$client->notice($nick, sprintf($first->language->NS_SELFDONE, "imposter"));	

	}
	
	/**
 * Ghost command to remove dead client connection
 *
 * @param string $nick
 * @param string $target
 * @param string $password
 * @return void
 */
	static function ghost($nick, $target, $password)
	{
		global $irc, $me, $language;
		
		if ((!$target) or (!$password))
			return self::$client->notice($nick, "SYNTAX: Ghost <username> <password>");
		
		$user = user::getuser($target);
		
		if (!$user)
			return self::$client->notice($nick, sprintf($language->CS_NOUSERONIRC, $target));
			
		if ($nick == $target)
			return self::$client->notice($nick, sprintf($user->NS_NOSELF, 'GHOST'));
			
		if ($user->password != (md5($user->secret . $password . $user->secret)))	
			return nickserv::denied($nick);
			
		/* passed all security */
		//:lynk KILL admin :smurfs.eu.torrentmax.org!lynk.coder.torrentmax.org!~lynk!lynk (adsasd)
		
		$irc->send(sprintf(KILL, 
			$me->nickserv['nick'], 
			$target, 
			$me->name, 
			$me->nickserv['host'], 
			$me->nickserv['username'],
			$me->nickserv['nick'],
			"Ghost command from {$nick}"));
			
		self::$client->notice($nick, sprintf($user->language->NS_SELFDONE, 'ghost'));
		
		user::remuser($user);
	
	}
	
	/**
	 * Allows for the remote IDENTIFY of a nickname
	 * for when someone has 2 active clients.
	 * 
	 * @param string $nick
	 * @param string $target
	 * @param string $password
	 * @return void
	 */
	static function login($nick, $target, $password)
	{
		global $irc, $me, $language;
		
		if (!$password)
			return self::$client->notice($nick, "Syntax: LOGIN <sitenick> <password>");
			
		$user = user::getuser($nick);
		$array = user::sql_single($target);
		$is_online = user::getuser($target);
	
		if (!is_array($array))
			return self::$client->notice($nick, sprintf($language->NS_NOTREGISTERED, $target));
			
		if ($user->id != 0)
			return self::$client->notice($nick, $language->NS_NICKISREGISTERED);
			
		if (($user->modes & UMODE_REGISTERED) > 0)
		{
			self::$client->notice($nick, $user->language->NS_ALREADYIDENTIFIED);
			return;
		}
		
		if ($array['enabled'] == 'no')
			return self::$client->notice($nick, sprintf($language->REGBUTDISABLED, $me->support_channel));
			
		$hash = md5($array['secret'] . $password . $array['secret']);
		
		if ($hash != $array['passhash'])
		{
			self::$client->notice($nick, $language->NS_FAILEDIDENTIFY);
			$irc->send(sprintf(OPERWALL, self::$client->nick, sprintf($language->NS_FAILEDLOGINTELL, $nick, $user->username, $user->host, $target)));
			
			if ($is_online instanceof user)
				$irc->send(sprintf(NOTICE, self::$client->nick, $target, sprintf($is_online->language->NS_FAILEDLOGIN, $nick, $user->username, $user->vhost)));
				
			return;
		}
		
		$user->level = $array['class'];
		$user->modes = ($user->modes | UMODE_REGISTERED);
		$user->remote = true;
		$user->language = new language(language::$short_long[$array['lang']]);
		
		self::$client->notice($nick, sprintf($language->NS_LOGINOK, $target));
		
		$irc->send(sprintf(RAW, ':'.$me->name.' ENCAP * CHGHOST '.$nick.' :'.$nick.'.remote.'.$me->userbot['host']));
		$irc->send(sprintf(RAW, ':'.$me->name.' ENCAP * SVSMODE '.$nick.' '.$user->ts .' +x'));
		
		if ($array['class'] > SITE_VIP)
			$irc->send(sprintf(OPERWALL, $me->nickserv['nick'], sprintf($language->NS_LOGINTELL, $nick, $user->username, $user->host, $target)));
			
		if ($is_online instanceof user)
			self::$client->notice($target, sprintf($is_online->language->NS_LOGINTELL, $nick, $user->username, $user->vhost, $target));
	}
	
	/**
 * Identify command
 *
 * @param string $nick
 * @param string $password
 * @return void
 */
	static function identify($nick, $password)
	{
		global $irc, $me, $db, $language;
		
		if (!$password)
			return self::$client->notice($nick, "Syntax: Identify <password>");
			
		$object = &user::getuser($nick);
		
		if ($object->id == 0)
			return self::notregistered($nick, 'IDENTIFY');
			
		if ($object->forbid == true)
			return self::$client->notice($nick, sprintf($object->language->REGBUTDISABLED, $me->support_channel));
			
		if (($object->modes & UMODE_REGISTERED) > 0)
		{
			self::$client->notice($nick, $object->language->NS_ALREADYIDENTIFIED);
			return;
		}
		
		$hash = md5($object->secret . $password . $object->secret);
		
		if ($hash != $object->password)
		{
			self::$client->notice($nick, $object->language->NS_FAILEDIDENTIFY);
			$irc->send(sprintf(OPERWALL, self::$client->nick, sprintf($language->NS_FAILEDIDENTIFYTELL, $object->nick, $object->username, $object->host)));
		
			/* If the user has failed 3 identify attempts in 5minutes, kill the user from the network --lynk */
			if ($object->failed_auths > 0)
			{
				// if its been longer than 5 minutes, reset the counter.
				if ((time() - $object->last_failed_auth) > 300)
				{
					$object->failed_auths == 1;
					$object->last_failed_auth = time();
					return;
				}
				
				/* check if this is third strike. */
				if ($object->failed_auths == 2)
				{
					$irc->send(sprintf(KILL, 
						$me->nickserv['nick'], 
						$nick, 
						$me->name, 
						$me->nickserv['host'], 
						$me->nickserv['username'],
						$me->nickserv['nick'],
						$object->language->NS_FAILEDIDENTIFYKILL));
			
					$irc->send(sprintf(OPERWALL, self::$client->nick, sprintf($language->NS_FAILEDIDENTIFYKTELL, $object->nick, $object->username, $object->host)));
		
					user::remuser($object);
					$object->failed_auths == 0;
					return;
				}
			}
			
			$object->failed_auths++;
			$object->last_failed_auth = time();
										
			return;
		}
		
		else 
		{
			self::$client->notice($nick, $object->language->NS_IDENTIFYOK);
			$object->setwait(false);
			
			/* Staff members, get joined to the staff channel */
			if ($object->level > SITE_UPLOADER)
				$irc->send(sprintf(SVSJOIN,	$me->name, $nick, $me->staff_channel));
			
			$messages = new message($object->id);
			
			if ($messages->get_new(1) > 0)
				memoserv::$client->notice($nick, sprintf($object->language->MS_MSGWAITING, memoserv::$client->nick));
			
			if (isset(user::$levels[$object->level]))
				$host = user::$levels[$object->level];
				
			else 
				$host = 'user';
			
			/* Create a vhost object to find if we need to change a users host. */
			$vhost = new vhost();

			/* Check for a vhost on record for the current users id number. */
			if ($vhost->findById($object->id, 'userid'))
			{
				$irc->send(sprintf(RAW, ':'.$me->name.' ENCAP * CHGHOST '.$nick.' :'.$vhost->host));
				$irc->send(sprintf(RAW, ':'.$me->name.' ENCAP * SVSMODE '.$nick.' '.$object->ts .' +rx'));
				self::$client->notice($nick, sprintf($object->language->NS_VHOSTISNOW, $vhost->host));
				$object->vhost = $vhost->host;
				$object->modes = ($object->modes | UMODE_REGISTERED);
			}
			
			else 
			{	
				/* If its ME, the author, I dont want a generic virtualhost
				 * because I already have my special buddyserv.author.
				 */
				if (!$object->author)
				{
					$irc->send(sprintf(RAW, ':'.$me->name.' ENCAP * CHGHOST '.$nick.' :'.$nick.'.'.$host.'.'.$me->userbot['host']));
					$irc->send(sprintf(RAW, ':'.$me->name.' ENCAP * SVSMODE '.$nick.' '.$object->ts .' +rx'));
					
					$object->vhost = $nick.'.'.$host.'.'.$me->userbot['host'];
					$object->modes = ($object->modes | UMODE_REGISTERED);
					
					self::$client->notice($nick, 'Your virtual host has been activated. ('.$nick.'.'.$host.'.'.$me->userbot['host'].')');
				}
			}
			
			if ($object->level > SITE_MODERATOR)
				$irc->send(sprintf(RAW, ':'.$me->name.' ENCAP * SVSMODE '.$nick.' '.$object->ts .' +wzlskN'));
			/* We are going to go through the channels that the user is current on
			 * and give them their allowed status's automatically after identifying */
			
			if (is_array($object->channels))
			{
				foreach ($object->channels as $chan => $null)
				{
					$obj = channel::get($chan);
					
					if ($object->level == SITE_ADMIN)
						$irc->send(sprintf(MODE, chanserv::$client->nick, $chan, "+ao {$nick} {$nick}"));
						
					elseif ($object->level > SITE_ADMIN)
						$irc->send(sprintf(MODE, chanserv::$client->nick, $chan, "+qo {$nick} {$nick}"));
					
					elseif ($obj->founder == $object->nick)
						$irc->send(sprintf(MODE, chanserv::$client->nick, $chan, "+qo {$nick} {$nick}"));
						
					elseif ($obj->has_access($nick, chanserv::ACCESS_SOP))
						$irc->send(sprintf(MODE, chanserv::$client->nick, $chan, "+ao {$nick} {$nick}"));
						
					elseif ($obj->has_access($nick, chanserv::ACCESS_AOP))
						$irc->send(sprintf(MODE, chanserv::$client->nick, $chan, "+o {$nick}"));
						
					elseif ($obj->has_access($nick, chanserv::ACCESS_HOP))
						$irc->send(sprintf(MODE, chanserv::$client->nick, $chan, "+h {$nick}"));
						
					elseif ($obj->has_access($nick, chanserv::ACCESS_VOP))
						$irc->send(sprintf(MODE, chanserv::$client->nick, $chan, "+v {$nick}"));
				}
			}
		}
	}
	
	/**
	 * Tells user they must be registered to issue said command
	 *
	 * @param string $nick
	 * @param string $command
	 */
	static function notregistered($nick, $command)
	{	
		global $language;
		
		self::$client->notice($nick, $language->NS_NOTREGISTERED);
	}
	
	/**
	 * Tells a user their account is disabled
	 *
	 * @param string $nick
	 */
	static function yourdisabled($nick)
	{
		$userObj = user::getuser($nick);
		
		self::$client->notice($nick, $userObj->language->REGBUTDISABLED, $me->support_channel);
	}
	
	/**
	 * Run through the list of users
	 * waiting for identification, that are
	 * kill-timed, and run the auth method
	 * on them again.
	 * 
	 * @return void
	 */
	static public function checkKillTimed()
	{
		if (count(user::$killtimed) > 0)
		{
			foreach (user::$killtimed as $user => $last)
			{
				if ((time() - $last) < 20)
					continue;
					
				$userObj = user::getuser($user);
				
				if (!($userObj instanceof user))
				{
					/* must have been killed or something, remove it */
					unset(user::$killtimed[$user]);
					continue;
				}
					
				if (!$userObj->iswait())
					continue;
					
				/* start the auth check */
				nickserv::authstart($user, $userObj->waitcount, true);
			}
		}
		
		pcntl_alarm(5);
	}
	
   /**
 	* Begins the authentication notices
 	*
 	* @param string $nick
 	* @param int $level
 	* @param bool $timed
 	*/
	static function authstart($nick,$level=1,$timed=false)
	{
		global $irc;
		
		$userObj = user::getuser($nick);
		
		if ($level == 1)
		{
			self::$client->notice($nick, $userObj->language->NS_REGNOTIFY1);
			self::$client->notice($nick, sprintf($userObj->language->NS_REGNOTIFY2, self::$client->nick));
			self::$client->notice($nick, $userObj->language->NS_REGNOTIFY3);
			
			if ($timed)
			{	
				/* userlist isnt accurate!!! */
				$userObj->setwait(true);
					
				self::$client->notice($nick, $userObj->language->NS_KILLTIMED1); 
				$userObj->waitcount = 2;
				return;
			}
			
			return;
		}
		
		elseif ($level == 2)
		{
			self::$client->notice($nick, $userObj->language->NS_KILLTIMED2); 		
			$userObj->waitcount++;
			$userObj->setwait(true);
			return;
		}

		elseif ($level == 3)
		{
			self::$client->notice($nick, $userObj->language->NS_KILLTIMED3); 	
			$userObj->waitcount++;
			$userObj->setwait(true);
			return;	
		}
		
		elseif ($level == 4)
		{
			self::$client->notice($nick, $userObj->language->NS_KILLTIMED4); 
					
			$time = time();
			$guest = 'Guest' . substr($time,(strlen($time) - 4)) .rand(0,100); 
					
			$userObj->setwait(false);
					
			return $irc->send(sprintf(SVSNICK, $nick, $userObj->ts, $guest, time()));
		}
	} 
}		


?>
