<?php

/**
 * BuddyServ IRC Services
 * 
 * OperServ operator command handler client
 * 
 * @name OperServ
 * @author Raymond Lynk <me@raymondj.ca>
 * @copyright (C) 2010, Raymond Lynk
 * @link http://buddyserv.raymondj.ca
 * @package Clients
 * @version $Id: Class.operserv.php 361 2009-12-18 06:57:40Z lynk $
 */

/*
 * 		 ______           __     __         _______                        
 * 		|   __ \.--.--.--|  |.--|  |.--.--.|     __|.-----.----.--.--.     
 * 		|   __ <|  |  |  _  ||  _  ||  |  ||__     ||  -__|   _|  |  |     
 * 		|______/|_____|_____||_____||___  ||_______||_____|__|  \___/      
 *  				    			   |_|                                 
 *                                                                         
 */

class operserv
{
	static $client;
	
	/**
	 * Initialize the current service client.
	 * 
	 * @param string $nick
	 * @param string $username
	 * @param string $host
	 * @param string $name
	 * @return void
	 */
	static function initialize($nick, $username, $host, $name, $modes)
	{
		self::$client = new serviceclient();
		self::$client->initialize($nick, $username, $host, $name, $modes);
	}
	
	/**
	 * Magic getter
	 * 
	 * @param string $what
	 * @return mixed
	 */
	function __get($what)
	{
		if (self::$client->$what)
			return self::$client->$what;
			
	}
	
	/**
	 * Introduce the client to the uplink server.
	 * 
	 * @return void
	 */
	static function introduce()
	{
		self::$client->introduce();
	}
	
	/**
	 * Called to parse any messages to OperServ
	 *
	 * @param string $nick
	 * @param string $arguments
	 * @return void
	 */
	static function command_parser($nick, $arguments)
	{
		$user = user::getuser($nick);
		
		if (($user->modes & UMODE_REGISTERED) == 0)
			return self::denied($nick, $arguments[0]);
	
		if (!strcasecmp($arguments[0], 'AKILL'))
			return self::cmd_gline($nick, $arguments);
			
		if (!strcasecmp($arguments[0], 'GLOBAL'))
			return self::cmd_global($nick, $arguments);
			
		if (!strcasecmp($arguments[0], 'SAVE'))
			return self::cmd_savetodb($nick);
			
		if (!strcasecmp($arguments[0], 'LEVEL'))
			return self::cmd_level($nick, (isset($arguments[1]) ? $arguments[1] : false));
			
		if (!strcasecmp($arguments[0], 'RESTART'))
			return self::cmd_restart($nick, $arguments);
			
		if (!strcasecmp($arguments[0], 'DIE'))
			return self::cmd_die($nick, $arguments);
			
		if (!strcasecmp($arguments[0], 'STATS'))
			return self::cmd_stats($nick);
			
		if (!strcasecmp($arguments[0], 'MODE'))
			return self::cmd_mode($nick, $arguments);
			
		if (!strcasecmp($arguments[0], 'UPDATE'))
			return self::cmd_update($nick);
			
		if (!strcasecmp($arguments[0], 'RAW'))
			return self::cmd_raw($nick, $arguments);
			
		if (!strcasecmp($arguments[0], 'SVSJOIN'))
			return self::cmd_svsjoin($nick, $arguments[1], $arguments[2]);
		
		if (!strcasecmp($arguments[0], 'SVSPART'))
			return self::cmd_svspart($nick, $arguments[1], $arguments[2]);
			
		if (!strcasecmp($arguments[0], 'SVSNICK'))
			return self::cmd_svsnick($nick, $arguments[1], $arguments[2]);
			
		if (!strcasecmp($arguments[0], 'IGNORE'))
			return self::cmd_ignore($nick, $arguments);
			
		if (!strcasecmp($arguments[0], 'HELP'))
		{
			array_shift($arguments);
			
			if (count($arguments) == 0)
				return self::$client->help($nick, 'operserv');
			
			return self::$client->help($nick, 'operserv', $arguments);
		}
		
		return operserv::unknowncmd($nick, $arguments[0]);
	}
	
	/**
	 * Process ignore command
	 * 
	 * @param string $nick
	 * @param array $arguments
	 * @return void
	 */
	public static function cmd_ignore($nick, $arguments)
	{
		global $language;
		
		$userObj = user::getuser($nick);
		
		if ($userObj->level < SITE_ADMIN)
			return self::denied($nick, 'IGNORE');
			
		if (!strcasecmp($arguments[1], 'DEL'))
		{
			if (!isset($arguments[2]))
				return self::$client->notice($nick, 'Syntax: IGNORE DEL <mask>');
				
			if (!ignore::exists($arguments[2]))
				return self::$client->notice($nick, sprintf($userObj->language->OS_IGNORE_NOEXIST, $arguments[2]));
				
			ignore::get($arguments[2])->delete();
			self::$client->notice($nick, sprintf($userObj->language->OS_IGNORE_DEL, $arguments[2]));
			return self::$client->operwall(sprintf($language->OS_IGNORE_DELTELL, $nick, $arguments[2]));
			
		}
		
		elseif (!strcasecmp($arguments[1], 'ADD'))
		{
			array_shift($arguments);
			array_shift($arguments);
			$target = array_shift($arguments);
			$reason = implode(' ', $arguments);

			if (!(($target) or ($reason)))
				return self::$client->notice($nick, "Syntax: IGNORE ADD [mask] [reason]");
				
			if (!preg_match('/^\S+!\S+@\S+$/', $target))
				return self::$client->notice($nick, "Syntax: IGNORE ADD [nick!user@mask] [reason]");
				
			if (ignore::exists($target))
				return self::$client->notice($nick, sprintf($userObj->language->OS_IGNORE_EXISTS, $target));
				
			$ignore = new ignore();
			$ignore->mask = $target;
			$ignore->added = time();
			$ignore->expires = (time() + 7776000);
			$ignore->oper = $nick;
			$ignore->regex = make_wild($target);
			$ignore->reason = $reason;
			
			$ignore->save();
			
			self::$client->notice($nick, sprintf($userObj->language->OS_IGNORE_ADD, $target, '90 days', $reason));
			self::$client->operwall(sprintf($language->OS_IGNORE_ADDTELL, $nick, $target, '90 days', $reason));
			return;
		}
		
		elseif (!strcasecmp($arguments[1], 'LIST'))
		{
			self::$client->notice($nick, $userObj->language->OS_IGNORE_LIST);
			
			/* cycle each ignore, print out */
			foreach (ignore::$list as $mask => $instance)
			{
				self::$client->notice($nick, '['.$instance->id.'] '.$instance->mask);
				self::$client->notice($nick, $instance->reason);
			}
			
			return;
		}
		
		else
			return self::$client->notice(sprintf($userObj->language->CS_UNKNOWNCOMMAND, 'IGNORE '.$arguments[2]));
	}
	
	/**
	 * Forcibly change the nickname of a user
	 * on IRC to the specified new nickname. The
	 * new nickname must not be in use already or
	 * this command will fail.
	 * 
	 * @param string $nick
	 * @param string $target
	 * @param string $newnick
	 * @return void
	 */
	public static function cmd_svsnick($nick, $target, $newnick)
	{
		global $irc, $me, $language;
		
		$userObj = user::getuser($nick);
		
		if (!($target or $newnick))
			return self::$client->notice($nick, 'SYNTAX: SVSNICK <user> <newnick>');
			
		if ($userObj->level < SITE_SYSOP)
			return self::denied($nick, 'SVSNICK');
			
		/* check for the given user on the network */
		if (!($targetObj = user::getuser($target)))
			return self::$client->notice($nick, sprintf($userObj->language->CS_NOUSERONIRC, $target));
			
		/* check if the newnick is already on IRC */
		if (user::getuser($newnick))
			return self::$client->notice($nick, sprintf($userObj->language->OS_ONIRC, $target));
			
		/* Sanity checks */
		if (!preg_match('/^[A-Za-z0-9_-]{1,32}$/', $newnick))
			return self::$client->notice($nick, sprintf($userObj->language->OS_INVALIDNICK));
			
		self::$client->notice($nick, sprintf($userObj->language->OS_FORCENICKCHANGE, $target, $newnick));
		self::$client->notice($target, sprintf($userObj->language->OS_NICKCHANGED, $newnick));
		self::$client->operwall(sprintf($language->OS_NICKFORCEDTELL, $nick, $target, $newnick));
		
		return $irc->send(sprintf(SVSNICK, $target, $targetObj->ts, $newnick, time()));
	}
	
	/**
	 * Force a user to join the specified channel. This
	 * only works if the channel already exists. Attemting
	 * to join a user to a non-existent channel will fail.
	 * 
	 * @param string $nick
	 * @param string $target
	 * @param string $channel
	 * @return void
	 */
	static function cmd_svspart($nick, $target, $channel)
	{
		global $irc, $me, $language;
		
		if (!($target or $channel))
			return self::$client->notice($nick, 'SYNTAX: SVSPART <nick> <channel>');
			
		$userObj = user::getuser($nick);
		
		if ($userObj->level < SITE_SYSOP)
			return self::denied($nick, 'SVSPART');
			
		/* check if the user actually exists */
		if (!user::getuser($target))
			return self::$client->notice($nick, sprintf($userObj->language->CS_NOUSERONIRC, $target));
			
		/* check for existing channel. */
		if (!channel::exists($channel))
			return self::$client->notice($nick, sprintf($userObj->language->CS_NOSUCHCHANNEL, $channel));

		/* what if the user is already on the channel? */
		$chanObj = channel::get($channel);
		
		if (!$chanObj->isMember($target))
			return self::$client->notice($nick, sprintf($userObj->language->CS_NOUSERONCHAN, $target, $channel));
			
		self::$client->notice($nick, sprintf($userObj->language->OS_FORCEDPART, $target, $channel));
		self::$client->operwall(sprintf($language->OS_FORCEDPARTTELL, $nick, $target, $channel));
		
		/* now send the svsjoin */
		return $irc->send(sprintf(SVSPART, $me->name, $target, $channel));
	}
	
	/**
	 * Force a user to join the specified channel. This
	 * only works if the channel already exists. Attemting
	 * to join a user to a non-existent channel will fail.
	 * 
	 * @param string $nick
	 * @param string $target
	 * @param string $channel
	 * @return void
	 */
	static function cmd_svsjoin($nick, $target, $channel)
	{
		global $irc, $me;
		
		if (!($target or $channel))
			return self::$client->notice($nick, 'SYNTAX: SVSJOIN <nick> <channel>');
			
		$userObj = user::getuser($nick);
		
		if ($userObj->level < SITE_SYSOP)
			return self::denied($nick, 'SVSJOIN');
			
		/* check if the user actually exists */
		if (!user::getuser($target))
			return self::$client->notice($nick, sprintf($userObj->language->CS_NOUSERONIRC, $target));
			
		/* check for existing channel. */
		if (!channel::exists($channel))
			return self::$client->notice($nick, sprintf($userObj->language->CS_NOSUCHCHANNEL, $channel));

		/* what if the user is already on the channel? */
		$chanObj = channel::get($channel);
		
		if ($chanObj->isMember($target))
			return self::$client->notice($nick, sprintf($userObj->language->OS_ALREADYONCHANNEL, $target, $channel));
			
		self::$client->notice($nick, sprintf($userObj->language->OS_FORCEDJOIN, $target, $channel));
		self::$client->operwall(sprintf($language->OS_FORCEDJOINTELL, $nick, $target, $channel));
		
		/* now send the svsjoin */
		return $irc->send(sprintf(SVSJOIN, $me->name, $target, $channel));
	}
	
	/**
	 * Sends a RAW string to the IRC Server. This
	 * command only works if services are running
	 * in development state.
	 * 
	 * @param string $nick
	 * @param array $arguments
	 * @return void
	 */
	static function cmd_raw($nick, $arguments)
	{
		global $site, $irc;
		
		/* If not on development database, pretend this
		 * command doesn't exist.
		 */
		if (!$site)
			return self::unknowncmd($nick, 'RAW');
			
		$userObj = user::getuser($nick);
		
		if ($userObj->level < SITE_SYSOP)
			return self::denied($nick, 'RAW');
			
		array_shift($arguments);
		$rawstring = implode(' ', $arguments);
		
		$irc->send($rawstring);
		
		return self::$client->notice($nick, "RAW Sent: ". $rawstring);
		
	}
	
	/**
	 * Performs an SVN update of the current code.
	 * This ONLY works if current source is under
	 * SVN control.
	 * 
	 * @param string $nick
	 * @return bool
	 */
	static function cmd_update($nick)
	{
		global $me, $language;
		
		$user = user::getuser($nick);
		
		/* Only coders can do this. */
		if ((!$user or !$user->identified() or ($user->level < SITE_CODER)))
			return self::denied($nick, "UPDATE");
		
		/* Lets try a simple check to see if we are actually under svn control */
		if (!file_exists('.svn/'))
			return self::$client->notice($nick, $user->language->OS_NOTUNDERSVN);
			
		self::$client->operwall(sprintf($language->OS_SVNSTART, $nick));
		self::$client->msg($me->coders_channel, sprintf($language->OS_SVNSTART, $nick));
		
		exec('svn update', $result);
		
		/* if only 1 line, its usually just "at version 324" line for no updates */
		if (count($result) == 1)
		{
			self::$client->operwall($language->OS_SVN_NOUPDATES);
			return self::$client->msg($me->coders_channel, $language->OS_SVN_NOUPDATES);
		}	
		
		self::$client->operwall($language->OS_SVNRESTART);
		self::$client->msg($me->coders_channel, $language->OS_SVNRESPONSE);
		
		/* send each line of the update to the coders channel */
		foreach ($result as $line)
		{
			self::$client->msg($me->coders_channel, $line);
		}
		
		/* automatically call the restart command for the issueing user. */
		return self::cmd_restart($nick, array('blah', 'Automatic restart issued after code update.'));
	}		
	
	/**
	 * Command not known response
	 *
	 * @param string $nick
	 * @param string $command
	 * @return null
	 */
	static function unknowncmd($nick,$command)
	{	
		$user = user::getuser($nick);
		
		return self::$client->notice($nick, sprintf($user->language->CS_UNKNOWNCOMMAND, $command));
	}
	
	/**
	 * Changes a channel mode
	 * 
	 * @param string $nick
	 * @param array $arguments
	 * @return void
	 */
	static function cmd_mode($nick, $arguments)
	{
		global $irc, $language;
	
		array_shift($arguments);
		$channel = array_shift($arguments);
		$params = implode (' ', $arguments);
		$user = user::getuser($nick);
		
		if ((!$user or !$user->identified() or ($user->level < SITE_MODERATOR)))
			return self::denied($nick, implode(' ', $arguments));
			
		if (!$params)
			return self::$client->notice($nick, "Syntax: MODE #channel +ntv JOhn");
			
		self::$client->chanMode($channel, $params);
		
		self::$client->notice($nick, sprintf($user->language->OS_MODES_CHANGED, $channel, $params));
		$irc->send(sprintf(OPERWALL, self::$client->nick,  sprintf($language->OS_MODES_CHANGEDTELL, $nick, $channel,$params)));
	}
				
	/**
	 * Gets current receive/sent bandwidth
	 * 
	 * @param string $nick
	 * @return void
	 */
	static function cmd_stats($nick)
	{
		global $irc, $me, $pwd;
		
		$user = user::getuser($nick);
		
		if ((!$user or !$user->identified() or ($user->level < SITE_MODERATOR)))
			return self::denied($nick, 'STATS');
			
		$sendtx = convert_bytes($irc->sendtx);
		$recvrx = convert_bytes($irc->recvtx);
		$memory = convert_bytes(memory_get_usage());
		$verphp = phpversion();
		$uname = `uname -a`;
		$uptime = duration_long($me->started, time());
		
		self::$client->notice($nick, $user->language->OS_STATS_START);
		self::$client->notice($nick, sprintf($user->language->OS_STATS_VERSION, $me->version));
		self::$client->notice($nick, sprintf($user->language->OS_STATS_UPTIME, $uptime));
		self::$client->notice($nick, sprintf($user->language->OS_STATS_BANDWIDTH, "[Recvd: ".$recvrx['size'].$recvrx['byte']." | Sent: ".$sendtx['size'].$sendtx['byte'].']'));
		self::$client->notice($nick, sprintf($user->language->OS_STATS_MEMORY, $memory['size'].$memory['byte']));
		self::$client->notice($nick, "-");
		self::$client->notice($nick, sprintf($user->language->OS_STATS_OS, $uname));
		self::$client->notice($nick, sprintf($user->language->OS_STATS_DIRECTORY, $pwd));
		self::$client->notice($nick, sprintf($user->language->OS_STATS_PHP, $verphp));
		self::$client->notice($nick, $user->language->OS_STATS_END);
		
	}
	
	/**
	 * Restart BuddyServ command
	 * 
	 * @param string $nick
	 * @param array $arguments
	 * @return void
	 */
	static function cmd_restart($nick, $arguments)
	{
		global $irc, $me;
		
		$user = user::getuser($nick);
		
		if ((!$user or !$user->identified() or ($user->level < SITE_SYSOP)))
			return self::denied($nick, 'RESTART');
		
		array_shift($arguments);
		
		if (count($arguments) == 0)
			return self::$client->notice($nick, sprintf($user->language->OS_NOREASON, 'RESTART'));
			
		$reason = implode(' ', $arguments);
		
		/* first, run all our updates, or at least try... */
		$irc->send(sprintf(OPERWALL, self::$client->nick, "RESTART given by {$nick} [{$reason}]"));
		self::$client->notice($nick, "[restart] Performing database sync...");
		
		userbot::cmd_report_check();
		gline::run_expires();
		
		self::$client->notice($nick, "[restart] QUIT'ing service clients and restarting.");
		
		$irc->send(sprintf(QUIT, $me->nickserv['nick'], "restart from {$nick} ({$reason})"));
		$irc->send(sprintf(QUIT, userbot::$client->nick, "restart from {$nick} ({$reason})"));
		$irc->send(sprintf(QUIT, self::$client->nick, "restart from {$nick} ({$reason})"));
		$irc->send(sprintf(QUIT, $me->chanserv['nick'], "restart from {$nick} ({$reason})"));
		$irc->send(sprintf(QUIT, $me->monitor['nick'], "restart from {$nick} ({$reason})"));
		$irc->send(sprintf(QUIT, $me->memoserv['nick'], "restart from {$nick} ({$reason})"));
		$irc->send(sprintf(QUIT, $me->floodserv['nick'], "restart from {$nick} ({$reason})"));
		
		$irc->disconnect();
		restart();
	}
	
	/**
	 * Tells BuddyServ to exit without restart.
	 * 
	 * @param string $nick
	 * @param array $arguments
	 * @return void
	 */
	static function cmd_die($nick, $arguments)
	{
		global $irc, $me;
		
		$user = user::getuser($nick);
		
		if ((!$user or !$user->identified() or ($user->level < SITE_SYSOP)))
			return self::denied($nick, 'RESTART');
		
		array_shift($arguments);
		
		if (count($arguments) == 0)
			return self::$client->notice($nick, sprintf($user->language->OS_NOREASON, 'DIE'));
			
		$reason = implode(' ', $arguments);
		
		/* first, run all our updates, or at least try... */
		$irc->send(sprintf(OPERWALL, self::$client->nick, "DIE given by {$nick} [{$reason}]"));
		self::$client->notice($nick, "[die] Performing database sync...");
		
		userbot::cmd_report_check();
		gline::run_expires();
		
		self::$client->notice($nick, "[die] QUIT'ing service clients and exiting.");
		
		$irc->send(sprintf(QUIT, $me->nickserv['nick'], "die command from {$nick} (".$reason.")"));
		$irc->send(sprintf(QUIT, userbot::$client->nick, "die command from {$nick} (".$reason.")"));
		$irc->send(sprintf(QUIT, self::$client->nick, "die command from {$nick} (".$reason.")"));
		$irc->send(sprintf(QUIT, $me->chanserv['nick'], "die command from {$nick} (".$reason.")"));
		$irc->send(sprintf(QUIT, $me->monitor['nick'], "die command from {$nick} (".$reason.")"));
		$irc->send(sprintf(QUIT, $me->memoserv['nick'], "die command from {$nick} (".$reason.")"));
		$irc->send(sprintf(QUIT, $me->floodserv['nick'], "die command from {$nick} (".$reason.")"));
		
		$irc->send(sprintf(ERROR, $irc->host, 'Received DIE command from '.$nick));
		$irc->disconnect();
		
		/* remove pid file */
		unlink('.buddypid');
		
		exit;

	}
	
	/**
	 * Force database saves, usually before a QUIT
	 * 
	 * @param string $nick
	 * @return void
	 */
	static function cmd_savetodb($nick)
	{	
		/* No longer used. Most things are written
		 * directly to the database anyways... for now..
		 
		$user = user::getuser($nick);
		
		if ((!$user->identified()) or ($user->level < SITE_MODERATOR))
			return self::denied($nick, 'SAVE');
	
		self::$client->notice($nick, 'Running database update...');
		idle::check_dosave(false);
		self::$client->notice($nick, 'All that database stuff is done.');
		*/
	}
	
	/**
	 * Sends a message to all users on the irc network
	 * 
	 * @param string $nick
	 * @param array $message
	 * @return void
	 */
	static function cmd_global($nick,$message)
	{
		global $irc, $me;
		
		$user = user::getuser($nick);
		
		if (($user->level < SITE_ADMIN) or (!$user->identified()))
			return self::denied($nick, 'GLOBAL');
			
		array_shift($message);
		$message = implode(' ', $message);
		
		if (!$message)
			return $irc->send(sprintf(NOTICE, self::$client->nick, $nick, $user->language->OS_GLOBAL_NOMSG));
		
		$irc->send(sprintf(OPERWALL, self::$client->nick, "{$nick} uses GLOBAL command [".$message."]"));
		self::$client->notice('$$*.'.$me->userbot['host'], '[Global] '.$message);
		self::$client->notice($nick, $user->language->OS_GLOBAL_SENT);
	}
	
	/**
	 * Returns your current operserv level
	 * 
	 * @param string $nick
	 * @param string $who
	 * @return void
	 */
	static function cmd_level($nick, $who=false)
	{
		$who = ($who) ? $who : $nick;
		$user = user::getuser($who);
		$level_int = $user->level;
		$level_name = user::$levels[$level_int];
		
		self::$client->notice($nick, "{$who}'s OperServ level is {$level_int} [{$level_name}]");
	}
	 
	/**
	 * Gline command handler
	 * 
	 * @param string $nick
	 * @param array $arguments
	 * @return void
	 */
	static function cmd_gline($nick,$arguments)
	{
		global $irc, $me, $language;
		
		array_shift($arguments);
		$do = array_shift($arguments);
		$expire = array_shift($arguments);
		$mask = array_shift($arguments);
		$reason = implode(' ', $arguments);
		
		$nobj = user::getuser($nick);
		
		if ($nobj->level < SITE_MODERATOR)
			return self::denied($nick, 'AKill');
		
		if (!strcasecmp($do, 'LIST'))
		{
			$max = 200;
			
			if ($expire > 0)
			{
				/* make a 1000 max listing */
				if ($expire > 1000)
					$max = 1000;
					
				else 
					$max = $expire;
			}
			
			$glines_reversed = array_reverse(gline::$list);
			$i=0;
			
			self::$client->notice($nick, $nobj->language->OS_GLINE_START);
			
			foreach ($glines_reversed as $mask => $object)
			{
				if (!$object->mask)
					continue;
					
				$i++;
				
				/* check if max is reached */
				if ($i > $max)
					return self::$client->notice($nick, sprintf($nobj->language->OS_GLINE_LISTED, $max));
					
				$set = date("M/d/y", $object->added);
				
				if ($object->expires == 0)
					$expires = 'permanent';
				
				else	
					$expires = sprintf("%.2f", (($object->expires - current_time()) / 60));
					
				self::$client->notice($nick, "{$i}) [expires: {$expires} mins] ".$object->mask." by ".$object->oper);
				self::$client->notice($nick, "{$i}) Reason: ".$object->reason);
			}
			
			self::$client->notice($nick, sprintf($nobj->language->OS_GLINE_LISTED, $max));
			return;
		}
		
		/* Process a removal of a gline command */
		elseif (!strcasecmp($do, 'DEL'))
		{
			$mask = $expire;
			
			if (!$mask)
				return self::$client->notice($nick, 'SYNTAX: AKILL DEL <mask>');
				
				$gline = gline::get($mask);
				
				if (!$gline)
					return self::$client->notice($nick, sprintf($nobj->language->OS_GLINE_NOTFOUND, $mask));
				
				/* Remove the gline, put user/hostname parts in an array, and send the unkline to all servers */
				$gline = '';
				$userhost = explode('@', $mask);
				
				$irc->send(sprintf(UNKLINE, self::$client->nick, $userhost[0], $userhost[1]));
				
				if (gline::del($mask))
					return self::$client->notice($nick, sprintf($nobj->language->OS_GLINE_REMOVED, $mask));
				
				/* If we get here, it means our initial checks showed the gline, but it is no longer registered.
				 * This should not be possible, but better sane then severed */
				return self::$client->notice($nick, sprintf($nobj->language->OS_GLINE_DISAPPEARED, $mask));
		}
		
		elseif (!strcasecmp($do, 'ADD'))
		{
			if ((!$expire) and ($expire != 0))
				return self::$client->notice($nick, 'SYNTAX: AKILL ADD <+3(m|d|h|w)> <mask> <reason>');
				
			preg_match('/^\+(\d+)(\w{1})$/', $expire, $matches);
			
			if (is_array($matches))
			{
				$numeric = $matches[1];
				$duration = $matches[2];
			/* not a break */
				$userhost = explode('@', $mask);
				
				$time = 0;
				
				/* process the duration amount 3d 5w 1h */
				switch ($duration)
				{
					case 's':
						$time = $numeric;
						break;
						
					case 'm':
						$time = ($numeric * 60);
						break;
						
					case 'h':
						$time = ($numeric * 3600);
						break;
						
					case 'd':
					default:
						$time = ($numeric * 86400);
						break;
						
					case 'w':
						$time = ($numeric * 604800);
						break;
				} /* end switch */
				
				if ($time == 0)
					$expiretime = 0;
				
				else 
					$expiretime = (current_time() + $time);
					
				/* Check if we already exist in the database */
				$gline = gline::get($mask);
				
				if ($gline instanceof gline)
				{
					self::$client->notice($nick, sprintf($nobj->language->OS_GLINE_EXISTS, $mask));
					return;
				}
				
				$success = gline::create($expiretime, $nick, $mask, $reason);
				
				if ($success)
				{
					$gline = gline::get($mask);
					$total_ban = 0;
					/* cycle the user list, find all matching, and send byebye */
					
					$templist = user::$userlist;
					
					foreach ($templist as $name => $object)
					{
						$theusermask = $object->username .'@'. $object->host;
						
						
						/* found a match! */
						if (preg_match('/^'.$gline->regex.'$/', $theusermask))
						{
						
							/* if it matches an operator, ignore and tatle */
							if (($object->modes & UMODE_OPERATOR) > 0)
							{
								$irc->send(sprintf(OPERWALL, self::$client->nick, sprintf($language->OS_GLINE_OVERRIDE, $object->nick, $mask)));
								continue;
							}
						
							$total_ban++;
							
							$irc->send(sprintf(OPERWALL, self::$client->nick, 
								sprintf($language->OS_GLINE_ACTIVEFOR, $name, $gline->mask)));
								
							self::$client->notice($name, '*** Banned: '.$gline->reason);
							
							$istemp = (($expiretime) ? 'Temporary' : 'Permanent');
							 
							$irc->send(sprintf(KILL, 
								$me->name, 
								$name, 
								$me->name, 
								self::$client->host, 
								self::$client->username,
								$me->name,
								$istemp." K-Line (".$gline->reason.")"));
								
								user::remuser($name);
						}
					}
					
					/*tell the issuer */
					self::$client->notice($nick, 
						sprintf($nobj->language->OS_GLINE_ADDED, $gline->mask, $gline->reason));
							
					/* tell the opers */
					$irc->send(sprintf(OPERWALL, self::$client->nick,
						sprintf($language->OS_GLINETELL, $nick, $gline->mask, $gline-reason, (($expiretime) ? $numeric.$duration : 'Never'))));
						
					/* now add the kline to the servers */
	
					$irc->send(sprintf(KLINE, self::$client->nick, $time, $userhost[0], $userhost[1], $gline->reason));
					return;
				} // end success
				
				self::$client->notice($nick, 'There was a problem adding your AKill. Please contact a coder.');
				return;
					//$irc->send(sprintf(NOTICE, $me
			} /* end valid pregmatch */
			
			self::$client->notice($nick, 'SYNTAX: AKILL ADD|DEL|LIST <+3(m|d|h|w)> <mask> <reason>');
			return;
			
		} //end add
		
		else 
			return self::$client->notice($nick, 'SYNTAX: AKILL ADD|DEL|LIST <+3(m|d|h|w)> <mask> <reason>');
		
	} // end gline
	
	/**
	 * Function to send 'access denied' string
	 *
	 * @param string $nick
	 */
	static function denied($nick, $command="unknown")
	{
		global $irc, $me, $language;
		
		$user = user::getuser($nick);
		
		self::$client->notice($nick, $user->language->NS_ACCESSDENIED);
		$irc->send(sprintf(OPERWALL, self::$client->nick, sprintf($language->CS_DENIEDACCESSTELL, $command, $nick)));
	}
}

?>