<?php

/**
 * BuddyServ IRC Services
 * 
 * Raw socket handler used by the IRC class
 * to connect to a remote host.
 * 
 * @name Sockets
 * @author Raymond Lynk <me@raymondj.ca>
 * @copyright (C) 2010, Raymond Lynk
 * @link http://buddyserv.raymondj.ca
 * @package Core
 * @version $Id: Class.sockets.php 346 2009-12-01 08:00:33Z lynk $
 */

/*
 * 		 ______           __     __         _______                        
 * 		|   __ \.--.--.--|  |.--|  |.--.--.|     __|.-----.----.--.--.     
 * 		|   __ <|  |  |  _  ||  _  ||  |  ||__     ||  -__|   _|  |  |     
 * 		|______/|_____|_____||_____||___  ||_______||_____|__|  \___/      
 *  				    			   |_|                                 
 *                                                                         
 */

class sockets
{
	public $com;
	public $host;
	protected $port;
	protected $buffer;
	protected $size;
	protected $queue = array();
	protected $name;
	public $connected;
	public $sendtx = 0;
	public $recvtx = 0;
	
	/**
	 * Constructor to set host and port
	 * of our future connection
	 * 
	 * @param string $host
	 * @param int $port
	 * @return sockets
	 */
	public function __construct($host,$port)
	{
		$this->host = $host;
		$this->port = $port;
	}
	
	/**
	 * Connect to the server
	 * 
	 * @return bool
	 */
	final public function connect()
	{
		if (!$this->com)
			$this->com = fsockopen($this->host,$this->port);
		
		if (!$this->com)
		{
			$this->connected = false;
			return false;
		}
		
		/* Setting non-blocking stream is BAD for cpu */
		//stream_set_blocking($this->com, 0);
		stream_set_timeout($this->com, 4);
		
		$this->connected = true;	
		return true;
	}
	
	/**
	 * Send a string to the socket
	 * 
	 * @param string $string
	 * @return bool
	 */
	public function sendto($string)
	{
		if (!$this->connected)
			return false;
		
		$this->sendtx += strlen($string);
		fputs($this->com, $string, strlen($string));
		return true;
	}
}

?>
