<?php

/**
 * BuddyServ IRC Services
 * 
 * User class that holds session information
 * for users connecting to the IRC network.
 * Also contains commands to manipulate database
 * user accounts.
 * 
 * @name User
 * @author Raymond Lynk <me@raymondj.ca>
 * @copyright (C) 2010, Raymond Lynk
 * @link http://buddyserv.raymondj.ca
 * @package Core
 * @version $Id: Class.user.php 351 2009-12-09 07:50:36Z lynk $
 */

/*
 * 		 ______           __     __         _______                        
 * 		|   __ \.--.--.--|  |.--|  |.--.--.|     __|.-----.----.--.--.     
 * 		|   __ <|  |  |  _  ||  _  ||  |  ||__     ||  -__|   _|  |  |     
 * 		|______/|_____|_____||_____||___  ||_______||_____|__|  \___/      
 *  				    			   |_|                                 
 *                                                                         
 */

class user
{
	protected $author = false;
	protected $nick;
	protected $regdate;
	protected $username;
	protected $host;
	protected $name;
	protected $server;
	protected $modes;
	protected $password;
	protected $vhost;
	protected $email;
	protected $ip;
	protected $language;
	protected $ip_long;
	protected $channels = array();
	protected $level=0;
	protected $verified = false;
	protected $forbid=false;
	protected $remote=false;
	protected $ts;
	private $secret;
	protected $id=0;
	public $failed_auths=0;
	public $last_failed_auth;
	private $db;
	public static $userlist = array();
	public static $killtimed = array();
	public $waitcount = 0;
	public static $levels = array(
		'0'		=> 'user',
		'1'		=> 'poweruser',
		'2'		=> 'VIP',
		'3'		=> 'uploader',
		'4'		=> 'moderator',
		'5'		=> 'admin',
		'6'		=> 'sysop',
		'7'		=> 'coder');
	
	/**
	 * Returns amount of online users
	 * 
	 * @return int
	 */
	static function count_online()
	{
		return count(self::$userlist);
	}
	
	/**
	 * Returns amount of non-identified
	 * guests on irc.
	 * 
	 * @return int
	 */
	static function count_guests()
	{
		$guests=0;
		
		if (self::count_online() == 0)
			return 0;
			
		foreach (self::$userlist as $null => $each)
		{
			if (!$each->identified())
				$guests++;
		}
		
		return $guests;
	}
			
	/**
	 * Check if a username is a registered member
	 * 
	 * @param string $username
	 * @return bool
	 */
	static function ismember($username)
	{
		global $db;
		
		$query = $db->query("SELECT `id` FROM `users` WHERE `username` = '".mysql_real_escape_string($username)."' LIMIT 1");
		
		if ($db->num_rows($query))
			return true;
			
		return false; 
	}
	
	/**
	 * Gets an instance of the user class for
	 * the given nickname, or returns false.
	 * 
	 * @param string $username
	 * @return mixed
	 */
	static function getuser($username)
	{
		$key = strtolower($username);
		
		if (array_key_exists($key, user::$userlist))
		{
			$object = &user::$userlist[$key];
			
			if ($object instanceof user)
				return $object;
			
			else
				return false;
		}
		
		return false;
	}
	/**
	 * Set the permanent language for
	 * the current user.
	 * 
	 * @param language $language
	 * @return bool
	 */
	public function setLanguage(language $language)
	{
		global $db;
		
		/* ensure our short name exists since the database
		 * doesnt take long-form
		 */
		if (!$language->short)
			return false;
		
		$query = $db->query(sprintf("UPDATE `users` SET `language` = '%s' WHERE `id` = %d",
			mysql_real_escape_string($language->short),
			$this->id));
			
		$this->language = $language;
			
		
	}
	
	/**
	 *  Rename a user, probably from a NICK commad 
	 *  
	 *  @param string $nick
	 *  @param string $newname
	 *  @return void
	 */
	static function rename($nick, $newname)
	{
		if (strcasecmp($nick, $newname) == 0)
			return;
			
		$user = self::getuser($nick);
		
		/* We need to fix our channel user lists */
		foreach ($user->channels as $channel => $array)
		{
			$chan = channel::get($channel);
			$chan->rename_user($user->nick, $newname);
			
		}
		
		
		/* Do this before assigning new nick */
		$lc_current = strtolower($user->nick);
		$lc = strtolower($newname);
		
		$nickdata = $user->sql_single($newname);
		
		$user->modes = ($user->modes & ~UMODE_REGISTERED);
		
		if (is_array($nickdata))
		{
			$user->id = $nickdata['id'];
			$user->password = $nickdata['passhash'];
			$user->level = $nickdata['class'];
			$user->email = $nickdata['email'];
			$user->forbid = ($nickdata['enabled'] == 'yes' ? false : true);
			$user->regdate = $nickdata['added'];
			$user->secret = $nickdata['secret'];
			
			$langupper = strtoupper($nickdata['language']);
			
			/* check if its a valid language, if not, using default */
			if (array_key_exists($langupper, language::$short_long))
				$user->language = new language(language::$short_long[$langupper]);
				
			else
				$user->language = new language(language::$default);

		}
		
		else 
		{
			$user->id = 0;
			$user->password = '';
			$user->level =  '';
			$user->email =  '';
			$user->forbid = '';
			$user->regdate = '';
			$user->secret =  '';
		}
		
		$user->nick = $newname;

		/* since we are really only using a pointer to the proper userlist
		 * we need to make a clone of our object, otherwise when the original
		 * is removed, the second one will break */	
		user::adduser($user);
		unset(user::$userlist[$lc_current]);
	}
		
		
	/**
	 * Add a user to our userlist. Returns
	 * a pointer to the object, or false.
	 * 
	 * @param user $object
	 * @return mixed
	 */
	static function adduser($object)
	{
		if ($object instanceof user)
		{
			$key = strtolower($object->nick);
			user::$userlist[$key] = $object;
			
			$pointer = &user::$userlist[$key];
			return $pointer;
		}
		
		return false;
	}
	
	/**
	 * Remove a user from memory
	 * 
	 * @param user $object
	 * @return bool
	 */
	static function remuser($object)
	{
		if ($object instanceof user)
		{
			/* remove this user from any channels they may be in first */
			foreach ($object->channels as $name => $value)
			{
				$channel = channel::get($name);
				
				if (!$channel)
					continue;
					
				/* remove the user from the channels datalist */
				if ($channel->deluser($object->nick) == 0)
					channel::remove($channel);
			}
				
			$key = strtolower($object->nick);
			
			unset(user::$userlist[$key]);
			return true;
		}
		
	}
	
	/**
	 * Create a new user object
	 * 
	 * @param string $nick
	 * @param string $username
	 * @param string $host
	 * @param string $name
	 * @param string $server
	 * @param string $modes
	 * @return user
	 */
	public function __construct($nick, $username, $host, $name, $server, $modes)
	{
		global $db;
		
		$this->nick = $nick;
		$this->username = $username;
		$this->host = $host;
		$this->ip = gethostbyname($host);
		$this->server = $server;
		$this->modes = user::binary_modes($modes);
		$this->name = $name;
		$this->db = &$db;
		$this->vhost = '255.255.255.255';
		$this->ip_long = sprintf("%u", ip2long($this->ip));
		
		//echo 'IPLONG is '.$this->ip_long."\n\n";
		
		$nickdata = user::sql_single($nick);
		
		if (is_array($nickdata))
		{
			$this->id = $nickdata['id'];
			$this->password = $nickdata['passhash'];
			$this->level = $nickdata['class'];
			$this->email = $nickdata['email'];
			$this->forbid = ($nickdata['enabled'] == 'yes' ? false : true);
			$this->regdate = $nickdata['added'];
			$this->secret = $nickdata['secret'];
			
			/* Set the language */
			$langupper = strtoupper($nickdata['language']);
			
			/* check if its a valid language, if not, using default */
			if (array_key_exists($langupper, language::$short_long))
				$this->language = new language(language::$short_long[$langupper]);
				
			else
				$this->language = new language(language::$default);
			
		}
		
		else
		{
			$this->language = new language(language::$default);
		}
	}
	
	/**
	 * Not identified, timed kill queue.
	 * true to add, false to remove
	 * 
	 * @param bool $true
	 * @return void
	 */
	public function setwait($true)
	{
		$lower = strtolower($this->nick);
		
		if ($true)
		{
			user::$killtimed[$lower] = time();
		}
		
		else
		{
			$this->waitcount = 0;
			unset(user::$killtimed[$lower]);
		}
	}
	
	/**
	 * Is in the timed kill queue?
	 * 
	 * @return bool
	 */
	public function iswait()
	{
		$lower = strtolower($this->nick);
		
		if (array_key_exists($lower, user::$killtimed))
			return true;
			
		return false;
	}
	
	/**
	 * Disable a user permanently
	 * 
	 * @param string $by
	 * @param string $who
	 * @param string $reason
	 * @return bool
	 */
	public function disable($by,$who,$reason)
	{
		global $db;
		
		$who = mysql_real_escape_string($who);
		$date = strftime("%G-%M-%d", time());
		$reason = "{$date} - Disabled for: ".mysql_real_escape_string($reason);
		
		$query = $db->query("UPDATE `users` SET `enabled` = 'no', `modcomment` = CONCAT('".$reason.".\n', modcomment) WHERE `username` = '".$who."' LIMIT 1");
		
		if (@$db->num_affected($query) == 0)
			return false;
			
		return true;
	}
	
	/**
	 * Enables a user permanently
	 * 
	 * @param string $who
	 * @param string $reason
	 * @return bool
	 */
	public function enable($who,$reason)
	{
		global $db;
		
		$by = $who;
		$who = mysql_real_escape_string($who);
		$date = strftime("%G-%M-%d", time());
		$reason = "{$date} by {$by} ".mysql_real_escape_string($reason);
		
		$query = $db->query("UPDATE `users` SET `enabled` = 'yes', `modcomment` = CONCAT('".$reason.".\n', modcomment) WHERE `username` = '".$who."' LIMIT 1");
		
		if ($db->num_affected($query) == 0)
			return false;
			
		return true;
	}
	
	public function fullMask($vhost=false)
	{
		if ($vhost)
			$hostname = $this->vhost;
		
		else
			$hostname = $this->host;
			
		$mask = $this->nick.'!'.$this->username.'@'.$hostname;
		
		return $mask;
	}
	/**
	 * Gets a user from the database. Returns
	 * array or false on failure.
	 * 
	 * @param string $nick
	 * @return mixed
	 */
	static function sql_single($nick)
	{
		global $db;
		
		$query = $db->query("SELECT `id`,`username`,`ip`,`passhash`,`secret`,`email`,`status`,`added`,`enabled`,`class`, `donor`,`modcomment`,`language` FROM `users` WHERE `username` = '".mysql_real_escape_string($nick)."' LIMIT 1");
		
		if ($db->num_rows($query) == 0)
		{
			return false;
		}
		
		else 
		{
			$array = $db->fetch_array($query);
			return $array;
		}
	}
		
	/* Parses the given modes to their binary equivalent */
	static function binary_modes($modes)
	{
		global $usermodes;
		$binary = 0x00000;
		
		$each = str_split($modes);
		
		foreach ($each as $index => $mode)
		{
			if (($mode == '+') or ($mode == '-'))
				continue;
				
			if (array_key_exists($mode, $usermodes))
			{
			
				$define = $usermodes[$mode];
				$binary = ($binary | $define);
				
				continue;
			}
			
			continue;
		}
		
		return $binary;
		
	}
	
	/**
	 * User is identified?
	 * 
	 * @return bool
	 */
	public function identified()
	{
		if (($this->modes & UMODE_REGISTERED) > 0)
			return true;
			
		return false;
	}
	
	/**
	 * Add user to a channel
	 * 
	 * @param string $channel
	 * @param bit $binmode
	 * @return void
	 */
	public function addchannel($channel, $binmode)
	{
		$lchan = strtolower($channel);
		
		$this->channels[$lchan]['status'] = $binmode;
	}
	
	/**
	 * Remove a channel from our records.
	 * 
	 * @param string $channel
	 * @return void
	 */
	public function removeChannel($channel)
	{
		$lchan = strtolower($channel);
		
		if (array_key_exists($lchan, $this->channels))
			unset($this->channels[$lchan]);
	}
	
	/* Magic getter function */
	public function __get($what)
	{
		return (($this->{$what}) ? $this->{$what} : null);
	} 
	
	/* Magic setter function */
	public function __set($what, $value)
	{
		$this->{$what} = $value;
	}
	
	/**
	 * Verified account?
	 * 
	 * @return bool
	 */
	public function verified()
	{
		return $this->verified;
	}
}

?>
