<?php

/**
 * BuddyServ IRC Services
 * 
 * UserBot pseudo client used for generic site->irc
 * events and reporting.
 * 
 * @name UserBot
 * @author Raymond Lynk <me@raymondj.ca>
 * @copyright (C) 2010, Raymond Lynk
 * @link http://buddyserv.raymondj.ca
 * @package Clients
 * @version $Id: Class.userbot.php 355 2009-12-10 07:47:20Z lynk $
 */

/*
 * 		 ______           __     __         _______                        
 * 		|   __ \.--.--.--|  |.--|  |.--.--.|     __|.-----.----.--.--.     
 * 		|   __ <|  |  |  _  ||  _  ||  |  ||__     ||  -__|   _|  |  |     
 * 		|______/|_____|_____||_____||___  ||_______||_____|__|  \___/      
 *  				    			   |_|                                 
 *                                                                         
 */

class userbot
{
	static $username;
	static $host;
	static $name;
	static $nick;
	static $help_list = array();
	static $client;
	static $changes;

	/**
	 * Initialize the current service client.
	 * 
	 * @param string $nick
	 * @param string $username
	 * @param string $host
	 * @param string $name
	 * @return void
	 */
	static function initialize($nick, $username, $host, $name, $modes)
	{
		self::$client = new serviceclient();
		self::$client->initialize($nick, $username, $host, $name, $modes);
	}
	
	/**
	 * magic getter
	 * 
	 * @param string $what
	 * @return mixed
	 */
	function __get($what)
	{
		if (self::$client->$what)
			return self::$client->$what;
			
	}
	
	/**
	 * Introduce the client to the network
	 * @return unknown_type
	 */
	static function introduce()
	{
		self::$client->introduce();
	}
	
	/**
	 * Parse commands form a channel
	 * 
	 * @param string $nick
	 * @param string $channel
	 * @param array $arguments
	 * @return void
	 */
	static function command_parser($nick, $channel, $arguments)
	{
		$checkuser = user::getuser($nick);
		
		if ($checkuser->forbid)
			return self::$client->notice($nick, $checkuser->language->CS_DISABLEDACCOUNT);
			
		if (!$checkuser->identified())
			return self::$client->notice($nick, sprintf($checkuser->language->CS_NOTIDENTIFIED, nickserv::$client->nick));
			
		if (!strcasecmp($arguments[0], '-user'))
			return self::cmd_user($nick, $channel, $arguments);
	
		if (!strcasecmp($arguments[0], '-next'))
			return self::cmd_tvrage_next($nick, $channel, $arguments);
			
		if (!strcasecmp($arguments[0], '-info'))
			return nickserv::info($nick, $arguments[1]); 
			
		if (!strcasecmp($arguments[0], '-connectable'))
			return self::connectable($nick, $channel, $arguments[1]); 
			
		if (!strcasecmp($arguments[0], '-changes'))
			return self::changes($nick);
			
		if (!strcasecmp($arguments[0], '-syslog'))
			return self::cmd_syslog($nick, $channel, $arguments[1]);
			
		if (!strcasecmp($arguments[0], '-help'))
			$arguments[0] = 'HELP';
		
		if (!strcasecmp($arguments[0], 'HELP'))
		{
			array_shift($arguments);
			
			if (count($arguments) == 0)
				return self::$client->help($nick, 'userbot');
			
			return self::$client->help($nick, 'userbot', $arguments);
		}
	}
	
	/**
	 * Turns sysop logging on or off
	 * 
	 * @param string $nick
	 * @param string $channel
	 * @param on|off $switch
	 * @return mixed
	 */
	static function cmd_syslog($nick, $channel, $switch)
	{
		global $me;
		
		$user = user::getuser($nick);
		
		if (!$user->identified() or ($user->level < SITE_SYSOP))
			return false;
			
		if ($channel != $me->sysop_channel)
			return false;
			
		if ($switch == "on")
		{
			$me->syslog_verbose = true;
			return self::$client->msg($channel,"\0033Relaying syslog events.\003");
		}
		
		else 
		{
			$me->syslog_verbose = false;
			return self::$client->msg($channel, "\0034Syslog relaying halted.\003");
		}
			
	}
		
	/**
	 * Outputs changelog file
	 * 
	 * @param string $nick
	 * @return void
	 */
	static function changes($nick)
	{
		$user = user::getuser($nick);
		
		if ((!$user->identified()))
			return;
			
		if (!is_array(self::$changes))
		{
			self::$changes = explode("\r\n", file_get_contents('changes.txt'));
		}
		
		foreach (self::$changes as $line)
		{
			self::$client->notice($nick, $line);
		}
	}
	
	/**
	 * Performs user information command for site stats
	 * 
	 * @param string $nick
	 * @param string $channel
	 * @param array $arguments
	 * @return void
	 */
	static function cmd_user($nick, $channel, $arguments)
	{	
		$userObj = user::getuser($nick);
		
		if (isset($arguments[1]))
			$target = $arguments[1];
			
		else 
			$target = $nick;
			
		$infostring = self::user_stats_string($target);
		
		if (!$infostring)
			return self::$client->notice($nick, sprintf($userObj->language->NS_USERNOTREGISTERED, $target));
			
		return self::$client->msg($channel, $infostring);
	}
	
	/**
	 * Is a person connectable?
	 * 
	 * @param string $nick
	 * @param string $channel
	 * @param string $who
	 * @return void
	 */
	static function connectable($nick, $channel, $who)
	{
		global $db;
		
		$userObj = user::getuser($nick);
		
		if (!$who)
			$who = $nick;
			
		$query = $db->query("SELECT users.username,peers.connectable as connectable FROM `users` LEFT JOIN peers ON peers.userid=users.id WHERE users.username = '".mysql_real_escape_string($who)."' LIMIT 1");
		
		if ($db->num_rows($query) == 0)
			return self::$client->notice($nick, sprintf($userObj->language->NS_USERNOTREGISTERED, $who));
			
		$array = $db->fetch_array($query);
		
		if ($array['connectable'] == 'yes')
			return self::$client->msg($channel, "User {$who} is \0033CONNECTABLE\003");
			
		if ($array['connectable'] == 'no')
			return self::$client->msg($channel, "User {$who} is \0034UNCONNECTABLE\003");
			
		return self::$client->msg($channel, "I dont know if '{$who}' is connectable.");
	}
	
	/**
	 * Generates the user statistics string
	 * 
	 * @param string $who
	 * @return string
	 */
	static function user_stats_string($who)
	{
		global $db;
		
		$who = mysql_real_escape_string($who);
		
		$query = $db->query("SELECT `username`,`class`,`downloaded`,`uploaded`,`title` FROM `users` WHERE `username` = '".$who."' LIMIT 1");
		
		if ($db->num_rows($query) == 0)
			return false;
			
		$on_irc = user::getuser($who);
		
		if ($on_irc)
		{
			if ($on_irc->identified())
				$irc_status = 'Yes';
				
			else 
				$irc_status = 'No';
		}
		
		else 
			$irc_status = 'No';
			
		$array = $db->fetch_array($query);
		$level = $array['class'];
		
		if ($array['downloaded'] == 0)
			$ratio = 'inf';
		
		else 
			$ratio = sprintf("%.2f", $array['uploaded'] / $array['downloaded']);
			
		$alldown = convert_bytes($array['downloaded']);
		$allup = convert_bytes($array['uploaded']);
			
		$string = $array['username'].' - Class: ('.user::$levels[$level].") - Uploaded:\0033 ".$allup['size']."\003 ".$allup['byte']." - Downloaded:\0034 ".$alldown['size']."\003 ".$alldown['byte'].' - Ratio: ('.$ratio.') - OnIRC: ('.$irc_status.')';
		
		return $string;
	}
	
	/**
	 * Check for new torrents, users, reports, syslog, ect
	 * 
	 * @return void
	 */
	static function cmd_report_check()
	{
		global $db, $last_report_check, $me;
		
		/* Check for new torrents */
		//$query = $db->query("SELECT `id`, `save_as`, `size`, UNIX_TIMESTAMP(`added`) as `added` FROM `torrents` WHERE `added` > FROM_UNIXTIME({$last_report_check})");
		$query = $db->query("SELECT `id`, `save_as`, `size`, UNIX_TIMESTAMP(`added`) as `added` FROM `torrents` WHERE `added` > {$last_report_check}");
	
		if ($db->num_rows($query))
		{
		    while ($each = $db->fetch_array($query))
		    {
				$array = byte_convert($each['size']);
				$size = $array['size'];
				$byte = $array['byte'];
		
			    self::$client->msg($me->user_channel, "\2Torrent:\2 (".$size.$byte.")\0037 ".$each['save_as']."\003 - \00312".$me->url."/details.php?id=".$each['id']."\003");
			     
			}
		}
	
		/* End of new torrent notification */
		
		/* Begin New User notification */
		
	//	$query = $db->query("SELECT `id`,`email`,`added`,`ip`,`username` FROM `users` WHERE `added` > FROM_UNIXTIME({$last_report_check})");
		$query = $db->query("SELECT `id`,`email`,`added`,`ip`,`username` FROM `users` WHERE `added` > {$last_report_check}");
		if ($db->num_rows($query))
		{
			while ($each = $db->fetch_array($query))
			{
				$lines = array();
				/* check for double IP */
				$ip = $each['ip'];
				
				//	$invited = (($each['invitedby'] > 0) ? " \2Invited: \2\0033yes\003 " : ' ');
	
					$lines[] = "\2New User:\2 \0037".$each['username']."\003 \2Ip:\2 *.".substr($ip,(strlen($ip) - 6), strlen($ip))." \2Url:\2 \00312".$me->url."/userdetails.php?id=".$each['id']."\003";
			}
				
			foreach ($lines as $message)
			{
				self::$client->msg($me->staff_channel, $message);
			}
		}	
	}
	
	/**
	 * Retrieves next showtime (coded by ozatomic or TheFool)
	 * cant remember which.
	 * 
	 * @param string $nick
	 * @param string $channel
	 * @param array $arguments
	 * @return void
	 */
	static function cmd_tvrage_next($nick, $channel, $arguments)
	{	    
	    /* minor change to allow spaces in requests.
	     * automatically converts them to _ --lynk */
	    if (isset($arguments[1]))
	    {
	    	if (count($arguments) > 2)
	    	{
	    		array_shift($arguments);
	    		
	    		$show = implode('_', $arguments);
	    	}
	    	
	    	else 
	    		$show = $arguments[1];
	    		
		$fh = @fopen('http://www.tvrage.com/quickinfo.php?show='.$show, 'r');
		
		if($fh)
		{
		    $i = 0;
		    $lines = array();
		    
		    while (!feof($fh))
		    {
		        $i++;
				$buffer = fgets($fh, 4096);
				$buffer = explode('@', str_replace(array('^',"\n"), array(' ',''), $buffer));
				
			switch($buffer[0])
			{
		            case 'No Show Results Were Found For':
			        return self::$client->notice($nick, "{$show} Was not found");
				break;
			    case 'Show Name':
			        $lines[] = "Show:\0037 ".$buffer[1]."\003";
				break;
			    case 'Show URL':
			        $lines[] = "URL:\00312 ".$buffer[1]."\003";
				break;
			    case 'Latest Episode':
			        $lines[] = "Latest Ep:\0039 ".$buffer[1]."\003";
				break;
			    case 'Next Episode':
			        $lines[] = "Next Ep:\0037 ".$buffer[1]."\003";
				break;
			    case 'Status':
			        $lines[] = 'Status: '.$buffer[1];
				break;
			    case 'Airtime':
			        $lines[] = "Airtime:\0034 ".$buffer[1]."\003";
				break;
			   
			    case 'Premiered':
			    case 'RFC3339':
			    case 'Classification':
			    case 'Genres':
			    case 'Network':
			        default:
				break;
		       }
		   }
		   $string = implode(' :: ', $lines);
		   fclose($fh);
		   
		   if(!empty($string))
		       return self::$client->msg($channel, $string);
	       } else
	       {
	           return self::$client->msg($channel, "Connection to TVRage could not be made.");
	       }
	   }
	   return self::$client->msg($channel, "Usage: -next <show_name>");
	}
}
	
